#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: ce_ospf_vrf
version_added: "2.4"
short_description: Manages configuration of an OSPF VPN instance on HUAWEI CloudEngine switches.
description:
    - Manages configuration of an OSPF VPN instance on HUAWEI CloudEngine switches.
author: Yang yang (@CloudEngine Ansible)
options:
    ospf:
        description:
            - The ID of the ospf process.
              Valid values are an integer, 1 - 4294967295, the default value is 1.
        required: true
    route_id:
        description:
            - Specifies the ospf private route id,.
              Valid values are a string, formatted as an IP address
              (i.e. "10.1.1.1") the length is 0 - 20.
    vrf:
        description:
            - Specifies the vpn instance which use ospf,length is 1 - 31.
              Valid values are a string.
        default: _public_
    description:
        description:
            - Specifies the description information of ospf process.
    bandwidth:
        description:
            - Specifies the reference bandwidth used to assign ospf cost.
              Valid values are an integer, in Mbps, 1 - 2147483648, the default value is 100.
    lsaalflag:
        description:
            - Specifies the mode of timer to calculate interval of arrive LSA.
              If set the parameter but not specifies value, the default will be used.
              If true use general timer.
              If false use intelligent timer.
        type: bool
        default: 'no'
    lsaainterval:
        description:
            - Specifies the interval of arrive LSA when use the general timer.
              Valid value is an integer, in millisecond, from 0 to 10000.
    lsaamaxinterval:
        description:
            - Specifies the max interval of arrive LSA when use the intelligent timer.
              Valid value is an integer, in millisecond, from 0 to 10000, the default value is 1000.
    lsaastartinterval:
        description:
            - Specifies the start interval of arrive LSA when use the intelligent timer.
              Valid value is an integer, in millisecond, from 0 to 10000, the default value is 500.
    lsaaholdinterval:
        description:
            - Specifies the hold interval of arrive LSA when use the intelligent timer.
              Valid value is an integer, in millisecond, from 0 to 10000, the default value is 500.
    lsaointervalflag:
        description:
            - Specifies whether cancel the interval of LSA originate or not.
              If set the parameter but noe specifies value, the default will be used.
              true:cancel the interval of LSA originate, the interval is 0.
              false:do not cancel the interval of LSA originate.
        type: bool
        default: 'no'
    lsaointerval:
        description:
            - Specifies the interval of originate LSA .
              Valid value is an integer, in second, from 0 to 10, the default value is 5.
    lsaomaxinterval:
        description:
            - Specifies the max interval of originate LSA .
              Valid value is an integer, in millisecond, from 1 to 10000, the default value is 5000.
    lsaostartinterval:
        description:
            - Specifies the start interval of originate LSA .
              Valid value is an integer, in millisecond, from 0 to 1000, the default value is 500.
    lsaoholdinterval:
        description:
            - Specifies the hold interval of originate LSA .
              Valid value is an integer, in millisecond, from 0 to 5000, the default value is 1000.
    spfintervaltype:
        description:
            - Specifies the mode of timer which used to calculate SPF.
              If set the parameter but noe specifies value, the default will be used.
              If is intelligent-timer, then use intelligent timer.
              If is timer, then use second level timer.
              If is millisecond, then use millisecond level timer.
        choices: ['intelligent-timer','timer','millisecond']
        default: intelligent-timer
    spfinterval:
        description:
            - Specifies the interval to calculate SPF when use second level  timer.
              Valid value is an integer, in second, from 1 to 10.
    spfintervalmi:
        description:
            - Specifies the interval to calculate SPF when use millisecond level  timer.
              Valid value is an integer, in millisecond, from 1 to 10000.
    spfmaxinterval:
        description:
            - Specifies the max interval to calculate SPF when use intelligent timer.
              Valid value is an integer, in millisecond, from 1 to 20000, the default value is 5000.
    spfstartinterval:
        description:
            - Specifies the start interval to calculate SPF when use intelligent timer.
              Valid value is an integer, in millisecond, from 1 to 1000, the default value is 50.
    spfholdinterval:
        description:
            - Specifies the hold interval to calculate SPF when use intelligent timer.
              Valid value is an integer, in millisecond, from 1 to 5000, the default value is 200.
    state:
        description:
            - Specify desired state of the resource.
        choices: ['present', 'absent']
        default: present
'''

EXAMPLES = '''
- name: ospf vrf module test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:

  - name: Configure ospf route id
    ce_ospf_vrf:
      ospf: 2
      route_id: 2.2.2.2
      lsaointervalflag: False
      lsaointerval: 2
      provider: "{{ cli }}"
'''

RETURN = '''
proposed:
    description: k/v pairs of parameters passed into module
    returned: verbose mode
    type: dict
    sample: {
        "bandwidth": "100",
        "description": null,
        "lsaaholdinterval": "500",
        "lsaainterval": null,
        "lsaamaxinterval": "1000",
        "lsaastartinterval": "500",
        "lsaalflag": "False",
        "lsaoholdinterval": "1000",
        "lsaointerval": "2",
        "lsaointervalflag": "False",
        "lsaomaxinterval": "5000",
        "lsaostartinterval": "500",
        "process_id": "2",
        "route_id": "2.2.2.2",
        "spfholdinterval": "1000",
        "spfinterval": null,
        "spfintervalmi": null,
        "spfintervaltype": "intelligent-timer",
        "spfmaxinterval": "10000",
        "spfstartinterval": "500",
        "vrf": "_public_"
    }
existing:
    description: k/v pairs of existing configuration
    returned: verbose mode
    type: dict
    sample: {
                "bandwidthReference": "100",
                "description": null,
                "lsaArrivalFlag": "false",
                "lsaArrivalHoldInterval": "500",
                "lsaArrivalInterval": null,
                "lsaArrivalMaxInterval": "1000",
                "lsaArrivalStartInterval": "500",
                "lsaOriginateHoldInterval": "1000",
                "lsaOriginateInterval": "2",
                "lsaOriginateIntervalFlag": "false",
                "lsaOriginateMaxInterval": "5000",
                "lsaOriginateStartInterval": "500",
                "processId": "2",
                "routerId": "2.2.2.2",
                "spfScheduleHoldInterval": "1000",
                "spfScheduleInterval": null,
                "spfScheduleIntervalMillisecond": null,
                "spfScheduleIntervalType": "intelligent-timer",
                "spfScheduleMaxInterval": "10000",
                "spfScheduleStartInterval": "500",
                "vrfName": "_public_"
            }
end_state:
    description: k/v pairs of configuration after module execution
    returned: verbose mode
    type: dict
    sample: {
                "bandwidthReference": "100",
                "description": null,
                "lsaArrivalFlag": "false",
                "lsaArrivalHoldInterval": "500",
                "lsaArrivalInterval": null,
                "lsaArrivalMaxInterval": "1000",
                "lsaArrivalStartInterval": "500",
                "lsaOriginateHoldInterval": "1000",
                "lsaOriginateInterval": "2",
                "lsaOriginateIntervalFlag": "false",
                "lsaOriginateMaxInterval": "5000",
                "lsaOriginateStartInterval": "500",
                "processId": "2",
                "routerId": "2.2.2.2",
                "spfScheduleHoldInterval": "1000",
                "spfScheduleInterval": null,
                "spfScheduleIntervalMillisecond": null,
                "spfScheduleIntervalType": "intelligent-timer",
                "spfScheduleMaxInterval": "10000",
                "spfScheduleStartInterval": "500",
                "vrfName": "_public_"
            }
updates:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["ospf 2"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: boolean
    sample: False
'''

from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, ce_argument_spec

CE_NC_GET_OSPF_VRF = """
    <filter type="subtree">
      <ospfv2 xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <ospfv2comm>
          <ospfSites>
            <ospfSite>
              <processId></processId>
              <routerId></routerId>
              <vrfName></vrfName>
              <description></description>
              <bandwidthReference></bandwidthReference>
              <lsaArrivalFlag></lsaArrivalFlag>
              <lsaArrivalInterval></lsaArrivalInterval>
              <lsaArrivalMaxInterval></lsaArrivalMaxInterval>
              <lsaArrivalStartInterval></lsaArrivalStartInterval>
              <lsaArrivalHoldInterval></lsaArrivalHoldInterval>
              <lsaOriginateIntervalFlag></lsaOriginateIntervalFlag>
              <lsaOriginateInterval></lsaOriginateInterval>
              <lsaOriginateMaxInterval></lsaOriginateMaxInterval>
              <lsaOriginateStartInterval></lsaOriginateStartInterval>
              <lsaOriginateHoldInterval></lsaOriginateHoldInterval>
              <spfScheduleIntervalType></spfScheduleIntervalType>
              <spfScheduleInterval></spfScheduleInterval>
              <spfScheduleIntervalMillisecond></spfScheduleIntervalMillisecond>
              <spfScheduleMaxInterval></spfScheduleMaxInterval>
              <spfScheduleStartInterval></spfScheduleStartInterval>
              <spfScheduleHoldInterval></spfScheduleHoldInterval>
            </ospfSite>
          </ospfSites>
        </ospfv2comm>
      </ospfv2>
    </filter>
"""

CE_NC_CREATE_OSPF_VRF = """
      <ospfv2 xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <ospfv2comm>
          <ospfSites>
            <ospfSite operation="merge">
              <processId>%s</processId>
%s
              <vrfName>%s</vrfName>
              <description>%s</description>
              <bandwidthReference>%s</bandwidthReference>
              <lsaArrivalFlag>%s</lsaArrivalFlag>
              <lsaArrivalInterval>%s</lsaArrivalInterval>
              <lsaArrivalMaxInterval>%s</lsaArrivalMaxInterval>
              <lsaArrivalStartInterval>%s</lsaArrivalStartInterval>
              <lsaArrivalHoldInterval>%s</lsaArrivalHoldInterval>
              <lsaOriginateIntervalFlag>%s</lsaOriginateIntervalFlag>
              <lsaOriginateInterval>%s</lsaOriginateInterval>
              <lsaOriginateMaxInterval>%s</lsaOriginateMaxInterval>
              <lsaOriginateStartInterval>%s</lsaOriginateStartInterval>
              <lsaOriginateHoldInterval>%s</lsaOriginateHoldInterval>
              <spfScheduleIntervalType>%s</spfScheduleIntervalType>
              <spfScheduleInterval>%s</spfScheduleInterval>
              <spfScheduleIntervalMillisecond>%s</spfScheduleIntervalMillisecond>
              <spfScheduleMaxInterval>%s</spfScheduleMaxInterval>
              <spfScheduleStartInterval>%s</spfScheduleStartInterval>
              <spfScheduleHoldInterval>%s</spfScheduleHoldInterval>
            </ospfSite>
          </ospfSites>
        </ospfv2comm>
      </ospfv2>
"""
CE_NC_CREATE_ROUTE_ID = """
              <routerId>%s</routerId>
"""

CE_NC_DELETE_OSPF = """
      <ospfv2 xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <ospfv2comm>
          <ospfSites>
            <ospfSite operation="delete">
              <processId>%s</processId>
              <routerId>%s</routerId>
              <vrfName>%s</vrfName>
            </ospfSite>
          </ospfSites>
        </ospfv2comm>
      </ospfv2>
"""


def build_config_xml(xmlstr):
    """build_config_xml"""

    return '<config> ' + xmlstr + ' </config>'


class OspfVrf(object):
    """
    Manages configuration of an ospf instance.
    """

    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = None
        self.init_module()

        # module input info
        self.ospf = self.module.params['ospf']
        self.route_id = self.module.params['route_id']
        self.vrf = self.module.params['vrf']
        self.description = self.module.params['description']
        self.bandwidth = self.module.params['bandwidth']
        self.lsaalflag = self.module.params['lsaalflag']
        self.lsaainterval = self.module.params['lsaainterval']
        self.lsaamaxinterval = self.module.params['lsaamaxinterval']
        self.lsaastartinterval = self.module.params['lsaastartinterval']
        self.lsaaholdinterval = self.module.params['lsaaholdinterval']
        self.lsaointervalflag = self.module.params['lsaointervalflag']
        self.lsaointerval = self.module.params['lsaointerval']
        self.lsaomaxinterval = self.module.params['lsaomaxinterval']
        self.lsaostartinterval = self.module.params['lsaostartinterval']
        self.lsaoholdinterval = self.module.params['lsaoholdinterval']
        self.spfintervaltype = self.module.params['spfintervaltype']
        self.spfinterval = self.module.params['spfinterval']
        self.spfintervalmi = self.module.params['spfintervalmi']
        self.spfmaxinterval = self.module.params['spfmaxinterval']
        self.spfstartinterval = self.module.params['spfstartinterval']
        self.spfholdinterval = self.module.params['spfholdinterval']
        self.state = self.module.params['state']

        # ospf info
        self.ospf_info = dict()

        # state
        self.changed = False
        self.updates_cmd = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()
        self.lsa_arrival_changed = False
        self.lsa_originate_changed = False
        self.spf_changed = False
        self.route_id_changed = False
        self.bandwidth_changed = False
        self.description_changed = False
        self.vrf_changed = False

    def init_module(self):
        """" init module """

        self.module = AnsibleModule(
            argument_spec=self.spec, supports_check_mode=True)

    def check_response(self, xml_str, xml_name):
        """Check if response message is already succeed."""

        if "<ok/>" not in xml_str:
            self.module.fail_json(msg='Error: %s failed.' % xml_name)

    def is_valid_ospf_process_id(self):
        """check whether the input ospf process id is valid"""

        if not self.ospf.isdigit():
            return False
        if int(self.ospf) > 4294967295 or int(self.ospf) < 1:
            return False
        return True

    def is_valid_ospf_route_id(self):
        """check is ipv4 addr is valid"""

        if self.route_id.find('.') != -1:
            addr_list = self.route_id.split('.')
            if len(addr_list) != 4:
                return False
            for each_num in addr_list:
                if not each_num.isdigit():
                    return False
                if int(each_num) > 255:
                    return False
            return True
        return False

    def is_valid_vrf_name(self):
        """check whether the input ospf vrf name is valid"""

        if len(self.vrf) > 31 or len(self.vrf) < 1:
            return False
        if self.vrf.find('?') != -1:
            return False
        if self.vrf.find(' ') != -1:
            return False
        return True

    def is_valid_description(self):
        """check whether the input ospf description is valid"""

        if len(self.description) > 80 or len(self.description) < 1:
            return False
        if self.description.find('?') != -1:
            return False
        return True

    def is_valid_bandwidth(self):
        """check whether the input ospf bandwidth reference is valid"""

        if not self.bandwidth.isdigit():
            return False
        if int(self.bandwidth) > 2147483648 or int(self.bandwidth) < 1:
            return False
        return True

    def is_valid_lsa_arrival_interval(self):
        """check whether the input ospf lsa arrival interval is valid"""

        if self.lsaainterval is None:
            return False
        if not self.lsaainterval.isdigit():
            return False
        if int(self.lsaainterval) > 10000 or int(self.lsaainterval) < 0:
            return False
        return True

    def isvalidlsamaxarrivalinterval(self):
        """check whether the input ospf lsa max arrival interval is valid"""

        if not self.lsaamaxinterval.isdigit():
            return False
        if int(self.lsaamaxinterval) > 10000 or int(self.lsaamaxinterval) < 1:
            return False
        return True

    def isvalidlsastartarrivalinterval(self):
        """check whether the input ospf lsa start arrival interval is valid"""

        if not self.lsaastartinterval.isdigit():
            return False
        if int(self.lsaastartinterval) > 1000 or int(self.lsaastartinterval) < 0:
            return False
        return True

    def isvalidlsaholdarrivalinterval(self):
        """check whether the input ospf lsa hold arrival interval is valid"""

        if not self.lsaaholdinterval.isdigit():
            return False
        if int(self.lsaaholdinterval) > 5000 or int(self.lsaaholdinterval) < 0:
            return False
        return True

    def is_valid_lsa_originate_interval(self):
        """check whether the input ospf lsa originate interval is valid"""

        if not self.lsaointerval.isdigit():
            return False
        if int(self.lsaointerval) > 10 or int(self.lsaointerval) < 0:
            return False
        return True

    def isvalidlsaoriginatemaxinterval(self):
        """check whether the input ospf lsa originate max interval is valid"""

        if not self.lsaomaxinterval.isdigit():
            return False
        if int(self.lsaomaxinterval) > 10000 or int(self.lsaomaxinterval) < 1:
            return False
        return True

    def isvalidlsaostartinterval(self):
        """check whether the input ospf lsa originate start interval is valid"""

        if not self.lsaostartinterval.isdigit():
            return False
        if int(self.lsaostartinterval) > 1000 or int(self.lsaostartinterval) < 0:
            return False
        return True

    def isvalidlsaoholdinterval(self):
        """check whether the input ospf lsa originate hold interval is valid"""

        if not self.lsaoholdinterval.isdigit():
            return False
        if int(self.lsaoholdinterval) > 5000 or int(self.lsaoholdinterval) < 1:
            return False
        return True

    def is_valid_spf_interval(self):
        """check whether the input ospf spf interval is valid"""

        if not self.spfinterval.isdigit():
            return False
        if int(self.spfinterval) > 10 or int(self.spfinterval) < 1:
            return False
        return True

    def is_valid_spf_milli_interval(self):
        """check whether the input ospf spf millisecond level interval is valid"""

        if not self.spfintervalmi.isdigit():
            return False
        if int(self.spfintervalmi) > 10000 or int(self.spfintervalmi) < 1:
            return False
        return True

    def is_valid_spf_max_interval(self):
        """check whether the input ospf spf intelligent timer max interval is valid"""

        if not self.spfmaxinterval.isdigit():
            return False
        if int(self.spfmaxinterval) > 20000 or int(self.spfmaxinterval) < 1:
            return False
        return True

    def is_valid_spf_start_interval(self):
        """check whether the input ospf spf intelligent timer start interval is valid"""

        if not self.spfstartinterval.isdigit():
            return False
        if int(self.spfstartinterval) > 1000 or int(self.spfstartinterval) < 1:
            return False
        return True

    def is_valid_spf_hold_interval(self):
        """check whether the input ospf spf intelligent timer hold interval is valid"""

        if not self.spfholdinterval.isdigit():
            return False
        if int(self.spfholdinterval) > 5000 or int(self.spfholdinterval) < 1:
            return False
        return True

    def is_route_id_exist(self):
        """is route id exist"""

        if not self.ospf_info:
            return False

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] != self.ospf:
                continue
            if ospf_site["routerId"] == self.route_id:
                return True
            else:
                continue
        return False

    def get_exist_ospf_id(self):
        """get exist ospf process id"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["processId"]
            else:
                continue
        return None

    def get_exist_route(self):
        """get exist route id"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["routerId"]
            else:
                continue
        return None

    def get_exist_vrf(self):
        """get exist vrf"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["vrfName"]
            else:
                continue
        return None

    def get_exist_bandwidth(self):
        """get exist bandwidth"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["bandwidthReference"]
            else:
                continue
        return None

    def get_exist_lsa_a_interval(self):
        """get exist lsa arrival interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaArrivalInterval"]
            else:
                continue
        return None

    def get_exist_lsa_a_interval_flag(self):
        """get exist lsa arrival interval flag"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaArrivalFlag"]
            else:
                continue
        return None

    def get_exist_lsa_a_max_interval(self):
        """get exist lsa arrival max interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaArrivalMaxInterval"]
            else:
                continue
        return None

    def get_exist_lsa_a_start_interval(self):
        """get exist lsa arrival start interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaArrivalStartInterval"]
            else:
                continue
        return None

    def get_exist_lsa_a_hold_interval(self):
        """get exist lsa arrival hold interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaArrivalHoldInterval"]
            else:
                continue
        return None

    def getexistlsaointerval(self):
        """get exist lsa originate interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaOriginateInterval"]
            else:
                continue
        return None

    def getexistlsaointerval_flag(self):
        """get exist lsa originate interval flag"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaOriginateIntervalFlag"]
            else:
                continue
        return None

    def getexistlsaomaxinterval(self):
        """get exist lsa originate max interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaOriginateMaxInterval"]
            else:
                continue
        return None

    def getexistlsaostartinterval(self):
        """get exist lsa originate start interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaOriginateStartInterval"]
            else:
                continue
        return None

    def getexistlsaoholdinterval(self):
        """get exist lsa originate hold interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["lsaOriginateHoldInterval"]
            else:
                continue
        return None

    def get_exist_spf_interval(self):
        """get exist spf second level timer interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleInterval"]
            else:
                continue
        return None

    def get_exist_spf_milli_interval(self):
        """get exist spf millisecond level timer interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleIntervalMillisecond"]
            else:
                continue
        return None

    def get_exist_spf_max_interval(self):
        """get exist spf max interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleMaxInterval"]
            else:
                continue
        return None

    def get_exist_spf_start_interval(self):
        """get exist spf start interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleStartInterval"]
            else:
                continue
        return None

    def get_exist_spf_hold_interval(self):
        """get exist spf hold interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleHoldInterval"]
            else:
                continue
        return None

    def get_exist_spf_interval_type(self):
        """get exist spf hold interval"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["spfScheduleIntervalType"]
            else:
                continue
        return None

    def is_ospf_exist(self):
        """is ospf exist"""

        if not self.ospf_info:
            return False

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return True
            else:
                continue
        return False

    def get_exist_description(self):
        """is description exist"""

        if not self.ospf_info:
            return None

        for ospf_site in self.ospf_info["ospfsite"]:
            if ospf_site["processId"] == self.ospf:
                return ospf_site["description"]
            else:
                continue
        return None

    def check_params(self):
        """Check all input params"""

        if self.ospf == '':
            self.module.fail_json(
                msg='Error: The ospf process id should not be null.')
        if self.ospf:
            if not self.is_valid_ospf_process_id():
                self.module.fail_json(
                    msg='Error: The ospf process id should between 1 - 4294967295.')
        if self.route_id == '':
            self.module.fail_json(
                msg='Error: The ospf route id length should not be null.')
        if self.route_id:
            if not self.is_valid_ospf_route_id():
                self.module.fail_json(
                    msg='Error: The ospf route id length should between 0 - 20,i.e.10.1.1.1.')
        if self.vrf == '':
            self.module.fail_json(
                msg='Error: The ospf vpn instance length should not be null.')
        if self.vrf:
            if not self.is_valid_vrf_name():
                self.module.fail_json(
                    msg='Error: The ospf vpn instance length should between 0 - 31,but can not contain " " or "?".')
        if self.description == '':
            self.module.fail_json(
                msg='Error: The ospf description should not be null.')
        if self.description:
            if not self.is_valid_description():
                self.module.fail_json(
                    msg='Error: The ospf description length should between 1 - 80,but can not contain "?".')
        if self.bandwidth == '':
            self.module.fail_json(
                msg='Error: The ospf bandwidth reference should not be null.')
        if self.bandwidth:
            if not self.is_valid_bandwidth():
                self.module.fail_json(
                    msg='Error: The ospf bandwidth reference should between 1 - 2147483648.')
        if self.lsaalflag is True:
            if not self.is_valid_lsa_arrival_interval():
                self.module.fail_json(
                    msg='Error: The ospf lsa arrival interval should between 0 - 10000.')
            if self.lsaamaxinterval or self.lsaastartinterval or self.lsaaholdinterval:
                self.module.fail_json(
                    msg='Error: Non-Intelligent Timer and Intelligent Timer Interval of '
                        'lsa-arrival-interval can not configured at the same time.')
        if self.lsaalflag is False:
            if self.lsaainterval:
                self.module.fail_json(
                    msg='Error: The parameter of lsa arrival interval command is invalid, '
                    'because LSA arrival interval can not be config when the LSA arrival flag is not set.')
            if self.lsaamaxinterval == '' or self.lsaastartinterval == '' or self.lsaaholdinterval == '':
                self.module.fail_json(
                    msg='Error: The ospf lsa arrival intervals should not be null.')
            if self.lsaamaxinterval:
                if not self.isvalidlsamaxarrivalinterval():
                    self.module.fail_json(
                        msg='Error: The ospf lsa arrival max interval should between 1 - 10000.')
            if self.lsaastartinterval:
                if not self.isvalidlsastartarrivalinterval():
                    self.module.fail_json(
                        msg='Error: The ospf lsa arrival start interval should between 1 - 1000.')
            if self.lsaaholdinterval:
                if not self.isvalidlsaholdarrivalinterval():
                    self.module.fail_json(
                        msg='Error: The ospf lsa arrival hold interval should between 1 - 5000.')
        if self.lsaointervalflag is True:
            if self.lsaointerval or self.lsaomaxinterval \
                    or self.lsaostartinterval or self.lsaoholdinterval:
                self.module.fail_json(
                    msg='Error: Interval for other-type and Instantly Flag '
                        'of lsa-originate-interval can not configured at the same time.')
        if self.lsaointerval == '':
            self.module.fail_json(
                msg='Error: The ospf lsa originate interval should should not be null.')
        if self.lsaointerval:
            if not self.is_valid_lsa_originate_interval():
                self.module.fail_json(
                    msg='Error: The ospf lsa originate interval should between 0 - 10 s.')
        if self.lsaomaxinterval == '' or self.lsaostartinterval == '' or self.lsaoholdinterval == '':
            self.module.fail_json(
                msg='Error: The ospf lsa originate intelligent intervals should should not be null.')
        if self.lsaomaxinterval:
            if not self.isvalidlsaoriginatemaxinterval():
                self.module.fail_json(
                    msg='Error: The ospf lsa originate max interval should between 1 - 10000 ms.')
        if self.lsaostartinterval:
            if not self.isvalidlsaostartinterval():
                self.module.fail_json(
                    msg='Error: The ospf lsa originate start interval should between 0 - 1000 ms.')
        if self.lsaoholdinterval:
            if not self.isvalidlsaoholdinterval():
                self.module.fail_json(
                    msg='Error: The ospf lsa originate hold interval should between 1 - 5000 ms.')
        if self.spfintervaltype == '':
            self.module.fail_json(
                msg='Error: The ospf spf interval type should should not be null.')
        if self.spfintervaltype == 'intelligent-timer':
            if self.spfinterval is not None or self.spfintervalmi is not None:
                self.module.fail_json(
                    msg='Error: Interval second and interval millisecond '
                        'of spf-schedule-interval can not configured if use intelligent timer.')
            if self.spfmaxinterval == '' or self.spfstartinterval == '' or self.spfholdinterval == '':
                self.module.fail_json(
                    msg='Error: The ospf spf intelligent timer intervals should should not be null.')
            if self.spfmaxinterval and not self.is_valid_spf_max_interval():
                self.module.fail_json(
                    msg='Error: The ospf spf max interval of intelligent timer should between 1 - 20000 ms.')
            if self.spfstartinterval and not self.is_valid_spf_start_interval():
                self.module.fail_json(
                    msg='Error: The ospf spf start interval of intelligent timer should between 1 - 1000 ms.')
            if self.spfholdinterval and not self.is_valid_spf_hold_interval():
                self.module.fail_json(
                    msg='Error: The ospf spf hold interval of intelligent timer should between 1 - 5000 ms.')
        if self.spfintervaltype == 'timer':
            if self.spfintervalmi is not None:
                self.module.fail_json(
                    msg='Error: Interval second and interval millisecond '
                        'of spf-schedule-interval can not configured at the same time.')
            if self.spfmaxinterval or self.spfstartinterval or self.spfholdinterval:
                self.module.fail_json(
                    msg='Error: Interval second and interval intelligent '
                        'of spf-schedule-interval can not configured at the same time.')
            if self.spfinterval == '' or self.spfinterval is None:
                self.module.fail_json(
                    msg='Error: The ospf spf timer intervals should should not be null.')
            if not self.is_valid_spf_interval():
                self.module.fail_json(
                    msg='Error: Interval second should between 1 - 10 s.')
        if self.spfintervaltype == 'millisecond':
            if self.spfinterval is not None:
                self.module.fail_json(
                    msg='Error: Interval millisecond and interval second '
                        'of spf-schedule-interval can not configured at the same time.')
            if self.spfmaxinterval or self.spfstartinterval or self.spfholdinterval:
                self.module.fail_json(
                    msg='Error: Interval millisecond and interval intelligent '
                        'of spf-schedule-interval can not configured at the same time.')
            if self.spfintervalmi == '' or self.spfintervalmi is None:
                self.module.fail_json(
                    msg='Error: The ospf spf millisecond intervals should should not be null.')
            if not self.is_valid_spf_milli_interval():
                self.module.fail_json(
                    msg='Error: Interval millisecond should between 1 - 10000 ms.')

    def get_ospf_info(self):
        """ get the detail information of ospf """

        self.ospf_info["ospfsite"] = list()

        getxmlstr = CE_NC_GET_OSPF_VRF
        xml_str = get_nc_config(self.module, getxmlstr)
        if 'data/' in xml_str:
            return

        xml_str = xml_str.replace('\r', '').replace('\n', '').\
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

        root = ElementTree.fromstring(xml_str)

        # get the vpn address family and RD text
        ospf_sites = root.findall(
            "data/ospfv2/ospfv2comm/ospfSites/ospfSite")
        if ospf_sites:
            for ospf_site in ospf_sites:
                ospf_ele_info = dict()
                for ospf_site_ele in ospf_site:
                    if ospf_site_ele.tag in ["processId", "routerId", "vrfName", "bandwidthReference",
                                             "description", "lsaArrivalInterval", "lsaArrivalMaxInterval",
                                             "lsaArrivalStartInterval", "lsaArrivalHoldInterval", "lsaArrivalFlag",
                                             "lsaOriginateInterval", "lsaOriginateMaxInterval",
                                             "lsaOriginateStartInterval", "lsaOriginateHoldInterval",
                                             "lsaOriginateIntervalFlag", "spfScheduleInterval",
                                             "spfScheduleIntervalMillisecond", "spfScheduleMaxInterval",
                                             "spfScheduleStartInterval", "spfScheduleHoldInterval",
                                             "spfScheduleIntervalType"]:
                        ospf_ele_info[
                            ospf_site_ele.tag] = ospf_site_ele.text
                self.ospf_info["ospfsite"].append(ospf_ele_info)

    def get_proposed(self):
        """get proposed info"""

        self.proposed["process_id"] = self.ospf
        self.proposed["route_id"] = self.route_id
        self.proposed["vrf"] = self.vrf
        self.proposed["description"] = self.description
        self.proposed["bandwidth"] = self.bandwidth
        self.proposed["lsaalflag"] = self.lsaalflag
        self.proposed["lsaainterval"] = self.lsaainterval
        self.proposed["lsaamaxinterval"] = self.lsaamaxinterval
        self.proposed["lsaastartinterval"] = self.lsaastartinterval
        self.proposed["lsaaholdinterval"] = self.lsaaholdinterval
        self.proposed["lsaointervalflag"] = self.lsaointervalflag
        self.proposed["lsaointerval"] = self.lsaointerval
        self.proposed["lsaomaxinterval"] = self.lsaomaxinterval
        self.proposed["lsaostartinterval"] = self.lsaostartinterval
        self.proposed["lsaoholdinterval"] = self.lsaoholdinterval
        self.proposed["spfintervaltype"] = self.spfintervaltype
        self.proposed["spfinterval"] = self.spfinterval
        self.proposed["spfintervalmi"] = self.spfintervalmi
        self.proposed["spfmaxinterval"] = self.spfmaxinterval
        self.proposed["spfstartinterval"] = self.spfstartinterval
        self.proposed["spfholdinterval"] = self.spfholdinterval

    def operate_ospf_info(self):
        """operate ospf info"""

        config_route_id_xml = ''
        vrf = self.get_exist_vrf()
        if vrf is None:
            vrf = '_public_'
        description = self.get_exist_description()
        if description is None:
            description = ''
        bandwidth_reference = self.get_exist_bandwidth()
        if bandwidth_reference is None:
            bandwidth_reference = '100'
        lsa_in_interval = self.get_exist_lsa_a_interval()
        if lsa_in_interval is None:
            lsa_in_interval = ''
        lsa_arrival_max_interval = self.get_exist_lsa_a_max_interval()
        if lsa_arrival_max_interval is None:
            lsa_arrival_max_interval = '1000'
        lsa_arrival_start_interval = self.get_exist_lsa_a_start_interval()
        if lsa_arrival_start_interval is None:
            lsa_arrival_start_interval = '500'
        lsa_arrival_hold_interval = self.get_exist_lsa_a_hold_interval()
        if lsa_arrival_hold_interval is None:
            lsa_arrival_hold_interval = '500'
        lsa_originate_interval = self.getexistlsaointerval()
        if lsa_originate_interval is None:
            lsa_originate_interval = '5'
        lsa_originate_max_interval = self.getexistlsaomaxinterval()
        if lsa_originate_max_interval is None:
            lsa_originate_max_interval = '5000'
        lsa_originate_start_interval = self.getexistlsaostartinterval()
        if lsa_originate_start_interval is None:
            lsa_originate_start_interval = '500'
        lsa_originate_hold_interval = self.getexistlsaoholdinterval()
        if lsa_originate_hold_interval is None:
            lsa_originate_hold_interval = '1000'
        spf_interval = self.get_exist_spf_interval()
        if spf_interval is None:
            spf_interval = ''
        spf_interval_milli = self.get_exist_spf_milli_interval()
        if spf_interval_milli is None:
            spf_interval_milli = ''
        spf_max_interval = self.get_exist_spf_max_interval()
        if spf_max_interval is None:
            spf_max_interval = '5000'
        spf_start_interval = self.get_exist_spf_start_interval()
        if spf_start_interval is None:
            spf_start_interval = '50'
        spf_hold_interval = self.get_exist_spf_hold_interval()
        if spf_hold_interval is None:
            spf_hold_interval = '200'

        if self.route_id:
            if self.state == 'present':
                if self.route_id != self.get_exist_route():
                    self.route_id_changed = True
                    config_route_id_xml = CE_NC_CREATE_ROUTE_ID % self.route_id
            else:
                if self.route_id != self.get_exist_route():
                    self.module.fail_json(
                        msg='Error: The route id %s is not exist.' % self.route_id)
                self.route_id_changed = True
                configxmlstr = CE_NC_DELETE_OSPF % (
                    self.ospf, self.get_exist_route(), self.get_exist_vrf())
                conf_str = build_config_xml(configxmlstr)

                recv_xml = set_nc_config(self.module, conf_str)
                self.check_response(recv_xml, "OPERATE_VRF_AF")
                self.changed = True
                return
        if self.vrf != '_public_':
            if self.state == 'present':
                if self.vrf != self.get_exist_vrf():
                    self.vrf_changed = True
                    vrf = self.vrf
            else:
                if self.vrf != self.get_exist_vrf():
                    self.module.fail_json(
                        msg='Error: The vrf %s is not exist.' % self.vrf)
                self.vrf_changed = True
                configxmlstr = CE_NC_DELETE_OSPF % (
                    self.ospf, self.get_exist_route(), self.get_exist_vrf())
                conf_str = build_config_xml(configxmlstr)
                recv_xml = set_nc_config(self.module, conf_str)
                self.check_response(recv_xml, "OPERATE_VRF_AF")
                self.changed = True
                return
        if self.bandwidth:
            if self.state == 'present':
                if self.bandwidth != self.get_exist_bandwidth():
                    self.bandwidth_changed = True
                    bandwidth_reference = self.bandwidth
            else:
                if self.bandwidth != self.get_exist_bandwidth():
                    self.module.fail_json(
                        msg='Error: The bandwidth %s is not exist.' % self.bandwidth)
                if self.get_exist_bandwidth() != '100':
                    self.bandwidth_changed = True
                    bandwidth_reference = '100'
        if self.description:
            if self.state == 'present':
                if self.description != self.get_exist_description():
                    self.description_changed = True
                    description = self.description
            else:
                if self.description != self.get_exist_description():
                    self.module.fail_json(
                        msg='Error: The description %s is not exist.' % self.description)
                self.description_changed = True
                description = ''

        if self.lsaalflag is False:
            lsa_in_interval = ''
            if self.state == 'present':
                if self.lsaamaxinterval:
                    if self.lsaamaxinterval != self.get_exist_lsa_a_max_interval():
                        self.lsa_arrival_changed = True
                        lsa_arrival_max_interval = self.lsaamaxinterval
                if self.lsaastartinterval:
                    if self.lsaastartinterval != self.get_exist_lsa_a_start_interval():
                        self.lsa_arrival_changed = True
                        lsa_arrival_start_interval = self.lsaastartinterval
                if self.lsaaholdinterval:
                    if self.lsaaholdinterval != self.get_exist_lsa_a_hold_interval():
                        self.lsa_arrival_changed = True
                        lsa_arrival_hold_interval = self.lsaaholdinterval
            else:
                if self.lsaamaxinterval:
                    if self.lsaamaxinterval != self.get_exist_lsa_a_max_interval():
                        self.module.fail_json(
                            msg='Error: The lsaamaxinterval %s is not exist.' % self.lsaamaxinterval)
                    if self.get_exist_lsa_a_max_interval() != '1000':
                        lsa_arrival_max_interval = '1000'
                        self.lsa_arrival_changed = True
                if self.lsaastartinterval:
                    if self.lsaastartinterval != self.get_exist_lsa_a_start_interval():
                        self.module.fail_json(
                            msg='Error: The lsaastartinterval %s is not exist.' % self.lsaastartinterval)
                    if self.get_exist_lsa_a_start_interval() != '500':
                        lsa_arrival_start_interval = '500'
                        self.lsa_arrival_changed = True
                if self.lsaaholdinterval:
                    if self.lsaaholdinterval != self.get_exist_lsa_a_hold_interval():
                        self.module.fail_json(
                            msg='Error: The lsaaholdinterval %s is not exist.' % self.lsaaholdinterval)
                    if self.get_exist_lsa_a_hold_interval() != '500':
                        lsa_arrival_hold_interval = '500'
                        self.lsa_arrival_changed = True
        else:
            if self.state == 'present':
                lsaalflag = "false"
                if self.lsaalflag is True:
                    lsaalflag = "true"
                if lsaalflag != self.get_exist_lsa_a_interval_flag():
                    self.lsa_arrival_changed = True
                    if self.lsaainterval is None:
                        self.module.fail_json(
                            msg='Error: The lsaainterval is not supplied.')
                    else:
                        lsa_in_interval = self.lsaainterval
                else:
                    if self.lsaainterval:
                        if self.lsaainterval != self.get_exist_lsa_a_interval():
                            self.lsa_arrival_changed = True
                            lsa_in_interval = self.lsaainterval
            else:
                if self.lsaainterval:
                    if self.lsaainterval != self.get_exist_lsa_a_interval():
                        self.module.fail_json(
                            msg='Error: The lsaainterval %s is not exist.' % self.lsaainterval)
                    self.lsaalflag = False
                    lsa_in_interval = ''
                    self.lsa_arrival_changed = True

        if self.lsaointervalflag is False:
            if self.state == 'present':
                if self.lsaomaxinterval:
                    if self.lsaomaxinterval != self.getexistlsaomaxinterval():
                        self.lsa_originate_changed = True
                        lsa_originate_max_interval = self.lsaomaxinterval
                if self.lsaostartinterval:
                    if self.lsaostartinterval != self.getexistlsaostartinterval():
                        self.lsa_originate_changed = True
                        lsa_originate_start_interval = self.lsaostartinterval
                if self.lsaoholdinterval:
                    if self.lsaoholdinterval != self.getexistlsaoholdinterval():
                        self.lsa_originate_changed = True
                        lsa_originate_hold_interval = self.lsaoholdinterval
                if self.lsaointerval:
                    if self.lsaointerval != self.getexistlsaointerval():
                        self.lsa_originate_changed = True
                        lsa_originate_interval = self.lsaointerval
            else:
                if self.lsaomaxinterval:
                    if self.lsaomaxinterval != self.getexistlsaomaxinterval():
                        self.module.fail_json(
                            msg='Error: The lsaomaxinterval %s is not exist.' % self.lsaomaxinterval)
                    if self.getexistlsaomaxinterval() != '5000':
                        lsa_originate_max_interval = '5000'
                        self.lsa_originate_changed = True
                if self.lsaostartinterval:
                    if self.lsaostartinterval != self.getexistlsaostartinterval():
                        self.module.fail_json(
                            msg='Error: The lsaostartinterval %s is not exist.' % self.lsaostartinterval)
                    if self.getexistlsaostartinterval() != '500':
                        lsa_originate_start_interval = '500'
                        self.lsa_originate_changed = True
                if self.lsaoholdinterval:
                    if self.lsaoholdinterval != self.getexistlsaoholdinterval():
                        self.module.fail_json(
                            msg='Error: The lsaoholdinterval %s is not exist.' % self.lsaoholdinterval)
                    if self.getexistlsaoholdinterval() != '1000':
                        lsa_originate_hold_interval = '1000'
                        self.lsa_originate_changed = True
                if self.lsaointerval:
                    if self.lsaointerval != self.getexistlsaointerval():
                        self.module.fail_json(
                            msg='Error: The lsaointerval %s is not exist.' % self.lsaointerval)
                    if self.getexistlsaointerval() != '5':
                        lsa_originate_interval = '5'
                        self.lsa_originate_changed = True
        else:
            if self.state == 'present':
                if self.getexistlsaointerval_flag() != 'true':
                    self.lsa_originate_changed = True
                    lsa_originate_interval = '5'
                    lsa_originate_max_interval = '5000'
                    lsa_originate_start_interval = '500'
                    lsa_originate_hold_interval = '1000'
            else:
                if self.getexistlsaointerval_flag() == 'true':
                    self.lsaointervalflag = False
                    self.lsa_originate_changed = True
        if self.spfintervaltype != self.get_exist_spf_interval_type():
            self.spf_changed = True
        if self.spfintervaltype == 'timer':
            if self.spfinterval:
                if self.state == 'present':
                    if self.spfinterval != self.get_exist_spf_interval():
                        self.spf_changed = True
                        spf_interval = self.spfinterval
                        spf_interval_milli = ''
                else:
                    if self.spfinterval != self.get_exist_spf_interval():
                        self.module.fail_json(
                            msg='Error: The spfinterval %s is not exist.' % self.spfinterval)
                    self.spfintervaltype = 'intelligent-timer'
                    spf_interval = ''
                    self.spf_changed = True
        if self.spfintervaltype == 'millisecond':
            if self.spfintervalmi:
                if self.state == 'present':
                    if self.spfintervalmi != self.get_exist_spf_milli_interval():
                        self.spf_changed = True
                        spf_interval_milli = self.spfintervalmi
                        spf_interval = ''
                else:
                    if self.spfintervalmi != self.get_exist_spf_milli_interval():
                        self.module.fail_json(
                            msg='Error: The spfintervalmi %s is not exist.' % self.spfintervalmi)
                    self.spfintervaltype = 'intelligent-timer'
                    spf_interval_milli = ''
                    self.spf_changed = True
        if self.spfintervaltype == 'intelligent-timer':
            spf_interval = ''
            spf_interval_milli = ''
            if self.spfmaxinterval:
                if self.state == 'present':
                    if self.spfmaxinterval != self.get_exist_spf_max_interval():
                        self.spf_changed = True
                        spf_max_interval = self.spfmaxinterval
                else:
                    if self.spfmaxinterval != self.get_exist_spf_max_interval():
                        self.module.fail_json(
                            msg='Error: The spfmaxinterval %s is not exist.' % self.spfmaxinterval)
                    if self.get_exist_spf_max_interval() != '5000':
                        self.spf_changed = True
                        spf_max_interval = '5000'
            if self.spfstartinterval:
                if self.state == 'present':
                    if self.spfstartinterval != self.get_exist_spf_start_interval():
                        self.spf_changed = True
                        spf_start_interval = self.spfstartinterval
                else:
                    if self.spfstartinterval != self.get_exist_spf_start_interval():
                        self.module.fail_json(
                            msg='Error: The spfstartinterval %s is not exist.' % self.spfstartinterval)
                    if self.get_exist_spf_start_interval() != '50':
                        self.spf_changed = True
                        spf_start_interval = '50'
            if self.spfholdinterval:
                if self.state == 'present':
                    if self.spfholdinterval != self.get_exist_spf_hold_interval():
                        self.spf_changed = True
                        spf_hold_interval = self.spfholdinterval
                else:
                    if self.spfholdinterval != self.get_exist_spf_hold_interval():
                        self.module.fail_json(
                            msg='Error: The spfholdinterval %s is not exist.' % self.spfholdinterval)
                    if self.get_exist_spf_hold_interval() != '200':
                        self.spf_changed = True
                        spf_hold_interval = '200'

        if not self.description_changed and not self.vrf_changed and not self.lsa_arrival_changed \
                and not self.lsa_originate_changed and not self.spf_changed \
                and not self.route_id_changed and not self.bandwidth_changed:
            self.changed = False
            return
        else:
            self.changed = True
        lsaointervalflag = "false"
        lsaalflag = "false"
        if self.lsaointervalflag is True:
            lsaointervalflag = "true"
        if self.lsaalflag is True:
            lsaalflag = "true"
        configxmlstr = CE_NC_CREATE_OSPF_VRF % (
            self.ospf, config_route_id_xml, vrf,
            description, bandwidth_reference, lsaalflag,
            lsa_in_interval, lsa_arrival_max_interval, lsa_arrival_start_interval,
            lsa_arrival_hold_interval, lsaointervalflag, lsa_originate_interval,
            lsa_originate_max_interval, lsa_originate_start_interval, lsa_originate_hold_interval,
            self.spfintervaltype, spf_interval, spf_interval_milli,
            spf_max_interval, spf_start_interval, spf_hold_interval)

        conf_str = build_config_xml(configxmlstr)
        recv_xml = set_nc_config(self.module, conf_str)
        self.check_response(recv_xml, "OPERATE_VRF_AF")

    def get_existing(self):
        """get existing info"""

        self.get_ospf_info()
        self.existing['ospf_info'] = self.ospf_info["ospfsite"]

    def set_update_cmd(self):
        """ set update command"""
        if not self.changed:
            return

        if self.state == 'present':
            if self.vrf_changed:
                if self.vrf != '_public_':
                    if self.route_id_changed:
                        self.updates_cmd.append(
                            'ospf %s router-id %s vpn-instance %s' % (self.ospf, self.route_id, self.vrf))
                    else:
                        self.updates_cmd.append(
                            'ospf %s vpn-instance %s ' % (self.ospf, self.vrf))
                else:
                    if self.route_id_changed:
                        self.updates_cmd.append(
                            'ospf %s router-id %s' % (self.ospf, self.route_id))
            else:
                if self.route_id_changed:
                    if self.vrf != '_public_':
                        self.updates_cmd.append(
                            'ospf %s router-id %s vpn-instance %s' % (self.ospf, self.route_id, self.get_exist_vrf()))
                    else:
                        self.updates_cmd.append(
                            'ospf %s router-id %s' % (self.ospf, self.route_id))
        else:
            if self.route_id_changed:
                self.updates_cmd.append('undo ospf %s' % self.ospf)
                return

        self.updates_cmd.append('ospf %s' % self.ospf)

        if self.description:
            if self.state == 'present':
                if self.description_changed:
                    self.updates_cmd.append(
                        'description %s' % self.description)
            else:
                if self.description_changed:
                    self.updates_cmd.append('undo description')
        if self.bandwidth_changed:
            if self.state == 'present':
                if self.get_exist_bandwidth() != '100':
                    self.updates_cmd.append(
                        'bandwidth-reference %s' % (self.get_exist_bandwidth()))
            else:
                self.updates_cmd.append('undo bandwidth-reference')
        if self.lsaalflag is True:
            if self.lsa_arrival_changed:
                if self.state == 'present':
                    self.updates_cmd.append(
                        'lsa-arrival-interval %s' % (self.get_exist_lsa_a_interval()))
                else:
                    self.updates_cmd.append(
                        'undo lsa-arrival-interval')

        if self.lsaalflag is False:
            if self.lsa_arrival_changed:
                if self.state == 'present':
                    if self.get_exist_lsa_a_max_interval() != '1000' \
                            or self.get_exist_lsa_a_start_interval() != '500'\
                            or self.get_exist_lsa_a_hold_interval() != '500':
                        self.updates_cmd.append('lsa-arrival-interval intelligent-timer %s %s %s'
                                                % (self.get_exist_lsa_a_max_interval(),
                                                   self.get_exist_lsa_a_start_interval(),
                                                   self.get_exist_lsa_a_hold_interval()))
                else:
                    if self.get_exist_lsa_a_max_interval() == '1000' \
                            and self.get_exist_lsa_a_start_interval() == '500'\
                            and self.get_exist_lsa_a_hold_interval() == '500':
                        self.updates_cmd.append(
                            'undo lsa-arrival-interval')
        if self.lsaointervalflag is False:
            if self.lsa_originate_changed:
                if self.state == 'present':
                    if self.getexistlsaointerval() != '5' \
                            or self.getexistlsaomaxinterval() != '5000' \
                            or self.getexistlsaostartinterval() != '500' \
                            or self.getexistlsaoholdinterval() != '1000':
                        self.updates_cmd.append('lsa-originate-interval other-type %s intelligent-timer %s %s %s'
                                                % (self.getexistlsaointerval(),
                                                   self.getexistlsaomaxinterval(),
                                                   self.getexistlsaostartinterval(),
                                                   self.getexistlsaoholdinterval()))
                else:
                    self.updates_cmd.append(
                        'undo lsa-originate-interval')
        if self.lsaointervalflag is True:
            if self.lsa_originate_changed:
                if self.state == 'present':
                    self.updates_cmd.append('lsa-originate-interval 0 ')
                else:
                    self.updates_cmd.append(
                        'undo lsa-originate-interval')
        if self.spfintervaltype == 'millisecond':
            if self.spf_changed:
                if self.state == 'present':
                    self.updates_cmd.append(
                        'spf-schedule-interval millisecond %s' % self.get_exist_spf_milli_interval())
                else:
                    self.updates_cmd.append(
                        'undo spf-schedule-interval')
        if self.spfintervaltype == 'timer':
            if self.spf_changed:
                if self.state == 'present':
                    self.updates_cmd.append(
                        'spf-schedule-interval %s' % self.get_exist_spf_interval())
                else:
                    self.updates_cmd.append(
                        'undo spf-schedule-interval')
        if self.spfintervaltype == 'intelligent-timer':
            if self.spf_changed:
                if self.state == 'present':
                    if self.get_exist_spf_max_interval() != '5000' \
                            or self.get_exist_spf_start_interval() != '50' \
                            or self.get_exist_spf_hold_interval() != '200':
                        self.updates_cmd.append('spf-schedule-interval intelligent-timer %s %s %s'
                                                % (self.get_exist_spf_max_interval(),
                                                   self.get_exist_spf_start_interval(),
                                                   self.get_exist_spf_hold_interval()))
                else:
                    self.updates_cmd.append(
                        'undo spf-schedule-interval')

    def get_end_state(self):
        """get end state info"""

        self.get_ospf_info()
        self.end_state['ospf_info'] = self.ospf_info["ospfsite"]

    def work(self):
        """worker"""

        self.check_params()
        self.get_existing()
        self.get_proposed()
        self.operate_ospf_info()
        self.get_end_state()
        self.set_update_cmd()
        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()

        self.module.exit_json(**self.results)


def main():
    """Module main"""

    argument_spec = dict(
        ospf=dict(required=True, type='str'),
        route_id=dict(required=False, type='str'),
        vrf=dict(required=False, type='str', default='_public_'),
        description=dict(required=False, type='str'),
        bandwidth=dict(required=False, type='str'),
        lsaalflag=dict(type='bool', default=False),
        lsaainterval=dict(required=False, type='str'),
        lsaamaxinterval=dict(required=False, type='str'),
        lsaastartinterval=dict(required=False, type='str'),
        lsaaholdinterval=dict(required=False, type='str'),
        lsaointervalflag=dict(type='bool', default=False),
        lsaointerval=dict(required=False, type='str'),
        lsaomaxinterval=dict(required=False, type='str'),
        lsaostartinterval=dict(required=False, type='str'),
        lsaoholdinterval=dict(required=False, type='str'),
        spfintervaltype=dict(required=False, default='intelligent-timer',
                             choices=['intelligent-timer', 'timer', 'millisecond']),
        spfinterval=dict(required=False, type='str'),
        spfintervalmi=dict(required=False, type='str'),
        spfmaxinterval=dict(required=False, type='str'),
        spfstartinterval=dict(required=False, type='str'),
        spfholdinterval=dict(required=False, type='str'),
        state=dict(required=False, choices=['present', 'absent'], default='present'),
    )

    argument_spec.update(ce_argument_spec)
    module = OspfVrf(argument_spec)
    module.work()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# (c) 2015, Darren Worrall <darren@iweb.co.uk>
# (c) 2015, René Moser <mail@renemoser.net>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible. If not, see <http://www.gnu.org/licenses/>.

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: cs_loadbalancer_rule
short_description: Manages load balancer rules on Apache CloudStack based clouds.
description:
    - Add, update and remove load balancer rules.
version_added: '2.0'
author:
    - "Darren Worrall (@dazworrall)"
    - "René Moser (@resmo)"
options:
  name:
    description:
      - The name of the load balancer rule.
    required: true
  description:
    description:
      - The description of the load balancer rule.
  algorithm:
    description:
      - Load balancer algorithm
      - Required when using C(state=present).
    choices: [ 'source', 'roundrobin', 'leastconn' ]
    default: 'source'
  private_port:
    description:
      - The private port of the private ip address/virtual machine where the network traffic will be load balanced to.
      - Required when using C(state=present).
      - Can not be changed once the rule exists due API limitation.
  public_port:
    description:
      - The public port from where the network traffic will be load balanced from.
      - Required when using C(state=present).
      - Can not be changed once the rule exists due API limitation.
    required: true
  ip_address:
    description:
      - Public IP address from where the network traffic will be load balanced from.
    required: true
    aliases: [ 'public_ip' ]
  open_firewall:
    description:
      - Whether the firewall rule for public port should be created, while creating the new rule.
      - Use M(cs_firewall) for managing firewall rules.
    type: bool
    default: 'no'
  cidr:
    description:
      - CIDR (full notation) to be used for firewall rule if required.
  protocol:
    description:
      - The protocol to be used on the load balancer
  project:
    description:
      - Name of the project the load balancer IP address is related to.
  state:
    description:
      - State of the rule.
    default: 'present'
    choices: [ 'present', 'absent' ]
  domain:
    description:
      - Domain the rule is related to.
  account:
    description:
      - Account the rule is related to.
  zone:
    description:
      - Name of the zone in which the rule should be created.
      - If not set, default zone is used.
extends_documentation_fragment: cloudstack
'''

EXAMPLES = '''
# Create a load balancer rule
- local_action:
    module: cs_loadbalancer_rule
    name: balance_http
    public_ip: 1.2.3.4
    algorithm: leastconn
    public_port: 80
    private_port: 8080

# update algorithm of an existing load balancer rule
- local_action:
    module: cs_loadbalancer_rule
    name: balance_http
    public_ip: 1.2.3.4
    algorithm: roundrobin
    public_port: 80
    private_port: 8080

# Delete a load balancer rule
- local_action:
    module: cs_loadbalancer_rule
    name: balance_http
    public_ip: 1.2.3.4
    state: absent
'''

RETURN = '''
---
id:
  description: UUID of the rule.
  returned: success
  type: string
  sample: a6f7a5fc-43f8-11e5-a151-feff819cdc9f
zone:
  description: Name of zone the rule is related to.
  returned: success
  type: string
  sample: ch-gva-2
project:
  description: Name of project the rule is related to.
  returned: success
  type: string
  sample: Production
account:
  description: Account the rule is related to.
  returned: success
  type: string
  sample: example account
domain:
  description: Domain the rule is related to.
  returned: success
  type: string
  sample: example domain
algorithm:
  description: Load balancer algorithm used.
  returned: success
  type: string
  sample: "source"
cidr:
  description: CIDR to forward traffic from.
  returned: success
  type: string
  sample: ""
name:
  description: Name of the rule.
  returned: success
  type: string
  sample: "http-lb"
description:
  description: Description of the rule.
  returned: success
  type: string
  sample: "http load balancer rule"
protocol:
  description: Protocol of the rule.
  returned: success
  type: string
  sample: "tcp"
public_port:
  description: Public port.
  returned: success
  type: string
  sample: 80
private_port:
  description: Private IP address.
  returned: success
  type: string
  sample: 80
public_ip:
  description: Public IP address.
  returned: success
  type: string
  sample: "1.2.3.4"
tags:
  description: List of resource tags associated with the rule.
  returned: success
  type: dict
  sample: '[ { "key": "foo", "value": "bar" } ]'
state:
  description: State of the rule.
  returned: success
  type: string
  sample: "Add"
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.cloudstack import (
    AnsibleCloudStack,
    cs_argument_spec,
    cs_required_together,
)


class AnsibleCloudStackLBRule(AnsibleCloudStack):

    def __init__(self, module):
        super(AnsibleCloudStackLBRule, self).__init__(module)
        self.returns = {
            'publicip': 'public_ip',
            'algorithm': 'algorithm',
            'cidrlist': 'cidr',
            'protocol': 'protocol',
        }
        # these values will be casted to int
        self.returns_to_int = {
            'publicport': 'public_port',
            'privateport': 'private_port',
        }

    def get_rule(self, **kwargs):
        rules = self.query_api('listLoadBalancerRules', **kwargs)
        if rules:
            return rules['loadbalancerrule'][0]

    def _get_common_args(self):
        return {
            'account': self.get_account(key='name'),
            'domainid': self.get_domain(key='id'),
            'projectid': self.get_project(key='id'),
            'zoneid': self.get_zone(key='id') if self.module.params.get('zone') else None,
            'publicipid': self.get_ip_address(key='id'),
            'name': self.module.params.get('name'),
        }

    def present_lb_rule(self):
        required_params = [
            'algorithm',
            'private_port',
            'public_port',
        ]
        self.module.fail_on_missing_params(required_params=required_params)

        args = self._get_common_args()
        rule = self.get_rule(**args)
        if rule:
            rule = self._update_lb_rule(rule)
        else:
            rule = self._create_lb_rule(rule)

        if rule:
            rule = self.ensure_tags(resource=rule, resource_type='LoadBalancer')
        return rule

    def _create_lb_rule(self, rule):
        self.result['changed'] = True
        if not self.module.check_mode:
            args = self._get_common_args()
            args.update({
                'algorithm': self.module.params.get('algorithm'),
                'privateport': self.module.params.get('private_port'),
                'publicport': self.module.params.get('public_port'),
                'cidrlist': self.module.params.get('cidr'),
                'description': self.module.params.get('description'),
                'protocol': self.module.params.get('protocol'),
            })
            res = self.query_api('createLoadBalancerRule', **args)

            poll_async = self.module.params.get('poll_async')
            if poll_async:
                rule = self.poll_job(res, 'loadbalancer')
        return rule

    def _update_lb_rule(self, rule):
        args = {
            'id': rule['id'],
            'algorithm': self.module.params.get('algorithm'),
            'description': self.module.params.get('description'),
        }
        if self.has_changed(args, rule):
            self.result['changed'] = True
            if not self.module.check_mode:
                res = self.query_api('updateLoadBalancerRule', **args)

                poll_async = self.module.params.get('poll_async')
                if poll_async:
                    rule = self.poll_job(res, 'loadbalancer')
        return rule

    def absent_lb_rule(self):
        args = self._get_common_args()
        rule = self.get_rule(**args)
        if rule:
            self.result['changed'] = True
        if rule and not self.module.check_mode:
            res = self.query_api('deleteLoadBalancerRule', id=rule['id'])

            poll_async = self.module.params.get('poll_async')
            if poll_async:
                self.poll_job(res, 'loadbalancer')
        return rule


def main():
    argument_spec = cs_argument_spec()
    argument_spec.update(dict(
        name=dict(required=True),
        description=dict(),
        algorithm=dict(choices=['source', 'roundrobin', 'leastconn'], default='source'),
        private_port=dict(type='int'),
        public_port=dict(type='int'),
        protocol=dict(),
        state=dict(choices=['present', 'absent'], default='present'),
        ip_address=dict(required=True, aliases=['public_ip']),
        cidr=dict(),
        project=dict(),
        open_firewall=dict(type='bool', default=False),
        tags=dict(type='list', aliases=['tag']),
        zone=dict(),
        domain=dict(),
        account=dict(),
        poll_async=dict(type='bool', default=True),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_together=cs_required_together(),
        supports_check_mode=True
    )

    acs_lb_rule = AnsibleCloudStackLBRule(module)

    state = module.params.get('state')
    if state in ['absent']:
        rule = acs_lb_rule.absent_lb_rule()
    else:
        rule = acs_lb_rule.present_lb_rule()

    result = acs_lb_rule.get_result(rule)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: ec2_group_facts
short_description: Gather facts about ec2 security groups in AWS.
description:
    - Gather facts about ec2 security groups in AWS.
version_added: "2.3"
requirements: [ boto3 ]
author: "Henrique Rodrigues (github.com/Sodki)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value. See \
      U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeSecurityGroups.html) for \
      possible filters. Filter names and values are case sensitive. You can also use underscores (_) \
      instead of dashes (-) in the filter keys, which will take precedence in case of conflict.
    required: false
    default: {}
notes:
  - By default, the module will return all security groups. To limit results use the appropriate filters.

extends_documentation_fragment:
    - aws
    - ec2
'''

EXAMPLES = '''
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Gather facts about all security groups
- ec2_group_facts:

# Gather facts about all security groups in a specific VPC
- ec2_group_facts:
    filters:
      vpc-id: vpc-12345678

# Gather facts about all security groups in a specific VPC
- ec2_group_facts:
    filters:
      vpc-id: vpc-12345678

# Gather facts about a security group
- ec2_group_facts:
    filters:
      group-name: example-1

# Gather facts about a security group by id
- ec2_group_facts:
    filters:
      group-id: sg-12345678

# Gather facts about a security group with multiple filters, also mixing the use of underscores as filter keys
- ec2_group_facts:
    filters:
      group_id: sg-12345678
      vpc-id: vpc-12345678

# Gather facts about various security groups
- ec2_group_facts:
    filters:
      group-name:
        - example-1
        - example-2
        - example-3

# Gather facts about any security group with a tag key Name and value Example. The quotes around 'tag:name' are important because of the colon in the value
- ec2_group_facts:
    filters:
      "tag:Name": Example
'''

RETURN = '''
security_groups:
    description: Security groups that match the provided filters. Each element consists of a dict with all the information related to that security group.
    type: list
    returned: always
    sample:
'''

import traceback

try:
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by imported HAS_BOTO3

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.ec2 import (ec2_argument_spec, boto3_conn, HAS_BOTO3, get_aws_connection_info,
                                      boto3_tag_list_to_ansible_dict, ansible_dict_to_boto3_filter_list,
                                      camel_dict_to_snake_dict)


def main():
    argument_spec = ec2_argument_spec()
    argument_spec.update(
        dict(
            filters=dict(default={}, type='dict')
        )
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    if not HAS_BOTO3:
        module.fail_json(msg='boto3 required for this module')

    region, ec2_url, aws_connect_params = get_aws_connection_info(module, boto3=True)

    if region:
        connection = boto3_conn(
            module,
            conn_type='client',
            resource='ec2',
            region=region,
            endpoint=ec2_url,
            **aws_connect_params
        )
    else:
        module.fail_json(msg="region must be specified")

    # Replace filter key underscores with dashes, for compatibility, except if we're dealing with tags
    sanitized_filters = module.params.get("filters")
    for key in sanitized_filters:
        if not key.startswith("tag:"):
            sanitized_filters[key.replace("_", "-")] = sanitized_filters.pop(key)

    try:
        security_groups = connection.describe_security_groups(
            Filters=ansible_dict_to_boto3_filter_list(sanitized_filters)
        )
    except ClientError as e:
        module.fail_json(msg=e.message, exception=traceback.format_exc())

    snaked_security_groups = []
    for security_group in security_groups['SecurityGroups']:
        # Modify boto3 tags list to be ansible friendly dict
        # but don't camel case tags
        security_group = camel_dict_to_snake_dict(security_group)
        security_group['tags'] = boto3_tag_list_to_ansible_dict(security_group.get('tags', {}), tag_name_key_name='key', tag_value_key_name='value')
        snaked_security_groups.append(security_group)

    module.exit_json(security_groups=snaked_security_groups)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = '''
---
module: nxos_smu
extends_documentation_fragment: nxos
version_added: "2.2"
short_description: Perform SMUs on Cisco NX-OS devices.
description:
    - Perform software maintenance upgrades (SMUs) on Cisco NX-OS devices.
author: Gabriele Gerbino (@GGabriele)
notes:
    - Tested against NXOSv 7.3.(0)D1(1) on VIRL
    - The module can only activate and commit a package,
      not remove or deactivate it.
    - Use C(transport=nxapi) to avoid connection timeout
options:
    pkg:
        description:
            - Name of the remote package.
        required: true
    file_system:
        description:
            - The remote file system of the device. If omitted,
              devices that support a file_system parameter will use
              their default values.
'''

EXAMPLES = '''
- nxos_smu:
    pkg: "nxos.CSCuz65185-n9k_EOR-1.0.0-7.0.3.I2.2d.lib32_n9000.rpm"
'''

RETURN = '''
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["install add bootflash:nxos.CSCuz65185-n9k_EOR-1.0.0-7.0.3.I2.2d.lib32_n9000.rpm",
             "install activate bootflash:nxos.CSCuz65185-n9k_EOR-1.0.0-7.0.3.I2.2d.lib32_n9000.rpm force",
             "install commit bootflash:nxos.CSCuz65185-n9k_EOR-1.0.0-7.0.3.I2.2d.lib32_n9000.rpm"]
'''


import collections
import re
import time

from ansible.module_utils.network.nxos.nxos import get_config, load_config, run_commands
from ansible.module_utils.network.nxos.nxos import nxos_argument_spec, check_args
from ansible.module_utils.basic import AnsibleModule


def execute_show_command(command, module):
    cmds = [{
        'command': command,
        'output': 'text',
    }]

    return run_commands(module, cmds)


def remote_file_exists(module, dst, file_system='bootflash:'):
    command = 'dir {0}/{1}'.format(file_system, dst)
    body = execute_show_command(command, module)
    if 'No such file' in body[0]:
        return False
    return True


def apply_patch(module, commands):
    for command in commands:
        load_config(module, [command])
        time.sleep(5)


def get_commands(module, pkg, file_system):
    commands = []
    splitted_pkg = pkg.split('.')
    fixed_pkg = '.'.join(splitted_pkg[0:-1])

    command = 'show install inactive'
    inactive_body = execute_show_command(command, module)

    command = 'show install active'
    active_body = execute_show_command(command, module)

    if fixed_pkg not in inactive_body[0] and fixed_pkg not in active_body[0]:
        commands.append('install add {0}{1}'.format(file_system, pkg))

    if fixed_pkg not in active_body[0]:
        commands.append('install activate {0}{1} force'.format(
            file_system, pkg))
    command = 'show install committed'
    install_body = execute_show_command(command, module)
    if fixed_pkg not in install_body[0]:
        commands.append('install commit {0}{1}'.format(file_system, pkg))

    return commands


def main():
    argument_spec = dict(
        pkg=dict(required=True),
        file_system=dict(required=False, default='bootflash:'),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    warnings = list()
    check_args(module, warnings)
    results = {'changed': False, 'commands': [], 'warnings': warnings}

    pkg = module.params['pkg']
    file_system = module.params['file_system']
    remote_exists = remote_file_exists(module, pkg, file_system=file_system)

    if not remote_exists:
        module.fail_json(
            msg="The requested package doesn't exist on the device"
        )

    commands = get_commands(module, pkg, file_system)
    if commands:
        results['changed'] = True
        if not module.check_mode:
            apply_patch(module, commands)
        if 'configure' in commands:
            commands.pop(0)
        results['commands'] = commands

    module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, VMware, Inc.
# Copyright: (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}


DOCUMENTATION = '''
---
module: vmware_maintenancemode
short_description: Place a host into maintenance mode
description:
    - This module can be used for placing a ESXi host into maintenance mode.
    - Support for VSAN compliant maintenance mode when selected.
author:
- Jay Jahns (@jjahns) <jjahns@vmware.com>
- Abhijeet Kasurde (@Akasurde)
version_added: "2.1"
notes:
    - Tested on vSphere 5.5, 6.0 and 6.5
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
    esxi_hostname:
        description:
            - Name of the host as defined in vCenter.
        required: True
    vsan:
        description:
            - Specify which VSAN compliant mode to enter.
        choices:
            - 'ensureObjectAccessibility'
            - 'evacuateAllData'
            - 'noAction'
        required: False
        aliases: [ 'vsan_mode' ]
    evacuate:
        description:
            - If set to C(True), evacuate all powered off VMs.
        default: False
        required: False
        type: bool
    timeout:
        description:
            - Specify a timeout for the operation.
        required: False
        default: 0
    state:
        description:
            - Enter or exit maintenance mode.
        choices:
            - present
            - absent
        default: present
        required: False
extends_documentation_fragment: vmware.documentation
'''

EXAMPLES = '''
- name: Enter VSAN-Compliant Maintenance Mode
  vmware_maintenancemode:
    hostname: vc_host
    username: vc_user
    password: vc_pass
    esxi_hostname: esxi.host.example
    vsan: ensureObjectAccessibility
    evacuate: yes
    timeout: 3600
    state: present
'''

RETURN = '''
hostsystem:
    description: Name of vim reference
    returned: always
    type: string
    sample: "'vim.HostSystem:host-236'"
hostname:
    description: Name of host in vCenter
    returned: always
    type: string
    sample: "esxi.local.domain"
status:
    description: Action taken
    returned: always
    type: string
    sample: "ENTER"
'''

try:
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vmware import PyVmomi, TaskError, vmware_argument_spec, wait_for_task
from ansible.module_utils._text import to_native


class VmwareMaintenanceMgr(PyVmomi):
    def __init__(self, module):
        super(VmwareMaintenanceMgr, self).__init__(module)
        self.esxi_hostname = self.module.params.get('esxi_hostname')
        self.vsan = self.module.params.get('vsan', None)
        self.host = self.find_hostsystem_by_name(host_name=self.esxi_hostname)
        if not self.host:
            self.module.fail_json(msg='Host %s not found in vCenter' % self.esxi_hostname)

    def EnterMaintenanceMode(self):
        if self.host.runtime.inMaintenanceMode:
            self.module.exit_json(changed=False,
                                  hostsystem=str(self.host),
                                  hostname=self.esxi_hostname,
                                  status='NO_ACTION',
                                  msg='Host %s already in maintenance mode' % self.esxi_hostname)

        spec = vim.host.MaintenanceSpec()

        if self.vsan:
            spec.vsanMode = vim.vsan.host.DecommissionMode()
            spec.vsanMode.objectAction = self.vsan

        try:
            task = self.host.EnterMaintenanceMode_Task(self.module.params['timeout'],
                                                       self.module.params['evacuate'],
                                                       spec)

            success, result = wait_for_task(task)

            self.module.exit_json(changed=success,
                                  hostsystem=str(self.host),
                                  hostname=self.esxi_hostname,
                                  status='ENTER',
                                  msg='Host %s entered maintenance mode' % self.esxi_hostname)

        except TaskError as e:
            self.module.fail_json(msg='Host %s failed to enter maintenance mode due to %s' % (self.esxi_hostname, to_native(e)))

    def ExitMaintenanceMode(self):
        if not self.host.runtime.inMaintenanceMode:
            self.module.exit_json(changed=False,
                                  hostsystem=str(self.host),
                                  hostname=self.esxi_hostname,
                                  status='NO_ACTION',
                                  msg='Host %s not in maintenance mode' % self.esxi_hostname)

        try:
            task = self.host.ExitMaintenanceMode_Task(self.module.params['timeout'])

            success, result = wait_for_task(task)

            self.module.exit_json(changed=success,
                                  hostsystem=str(self.host),
                                  hostname=self.esxi_hostname,
                                  status='EXIT',
                                  msg='Host %s exited maintenance mode' % self.esxi_hostname)
        except TaskError as e:
            self.module.fail_json(msg='Host %s failed to exit maintenance mode due to %s' % (self.esxi_hostname, to_native(e)))


def main():
    spec = vmware_argument_spec()
    spec.update(dict(esxi_hostname=dict(type='str', required=True),
                     vsan=dict(type='str',
                               choices=['ensureObjectAccessibility',
                                        'evacuateAllData',
                                        'noAction'],
                               aliases=['vsan_mode'],
                               ),
                     evacuate=dict(type='bool', default=False),
                     timeout=dict(default=0, type='int'),
                     state=dict(required=False, default='present', choices=['present', 'absent'])
                     )
                )

    module = AnsibleModule(argument_spec=spec)

    host_maintenance_mgr = VmwareMaintenanceMgr(module=module)

    if module.params['state'] == 'present':
        host_maintenance_mgr.EnterMaintenanceMode()
    elif module.params['state'] == 'absent':
        host_maintenance_mgr.ExitMaintenanceMode()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2013, Chatham Financial <oss@chathamfinancial.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: rabbitmq_vhost
short_description: Manage the state of a virtual host in RabbitMQ
description:
  - Manage the state of a virtual host in RabbitMQ
version_added: "1.1"
author: '"Chris Hoffman (@choffman)"'
options:
  name:
    description:
      - The name of the vhost to manage
    required: true
    aliases: [vhost]
  node:
    description:
      - erlang node name of the rabbit we wish to configure
    default: rabbit
    version_added: "1.2"
  tracing:
    description:
      - Enable/disable tracing for a vhost
    type: bool
    default: 'no'
    aliases: [trace]
  state:
    description:
      - The state of vhost
    default: present
    choices: [present, absent]
'''

EXAMPLES = '''
# Ensure that the vhost /test exists.
- rabbitmq_vhost:
    name: /test
    state: present
'''

from ansible.module_utils.basic import AnsibleModule


class RabbitMqVhost(object):
    def __init__(self, module, name, tracing, node):
        self.module = module
        self.name = name
        self.tracing = tracing
        self.node = node

        self._tracing = False
        self._rabbitmqctl = module.get_bin_path('rabbitmqctl', True)

    def _exec(self, args, run_in_check_mode=False):
        if not self.module.check_mode or (self.module.check_mode and run_in_check_mode):
            cmd = [self._rabbitmqctl, '-q', '-n', self.node]
            rc, out, err = self.module.run_command(cmd + args, check_rc=True)
            return out.splitlines()
        return list()

    def get(self):
        vhosts = self._exec(['list_vhosts', 'name', 'tracing'], True)

        for vhost in vhosts:
            if '\t' not in vhost:
                continue

            name, tracing = vhost.split('\t')
            if name == self.name:
                self._tracing = self.module.boolean(tracing)
                return True
        return False

    def add(self):
        return self._exec(['add_vhost', self.name])

    def delete(self):
        return self._exec(['delete_vhost', self.name])

    def set_tracing(self):
        if self.tracing != self._tracing:
            if self.tracing:
                self._enable_tracing()
            else:
                self._disable_tracing()
            return True
        return False

    def _enable_tracing(self):
        return self._exec(['trace_on', '-p', self.name])

    def _disable_tracing(self):
        return self._exec(['trace_off', '-p', self.name])


def main():
    arg_spec = dict(
        name=dict(required=True, aliases=['vhost']),
        tracing=dict(default='off', aliases=['trace'], type='bool'),
        state=dict(default='present', choices=['present', 'absent']),
        node=dict(default='rabbit'),
    )

    module = AnsibleModule(
        argument_spec=arg_spec,
        supports_check_mode=True
    )

    name = module.params['name']
    tracing = module.params['tracing']
    state = module.params['state']
    node = module.params['node']
    result = dict(changed=False, name=name, state=state)
    rabbitmq_vhost = RabbitMqVhost(module, name, tracing, node)

    if rabbitmq_vhost.get():
        if state == 'absent':
            rabbitmq_vhost.delete()
            result['changed'] = True
        else:
            if rabbitmq_vhost.set_tracing():
                result['changed'] = True
    elif state == 'present':
        rabbitmq_vhost.add()
        rabbitmq_vhost.set_tracing()
        result['changed'] = True

    module.exit_json(**result)

if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = """
---
module: eos_banner
version_added: "2.3"
author: "Peter Sprygada (@privateip)"
short_description: Manage multiline banners on Arista EOS devices
description:
  - This will configure both login and motd banners on remote devices
    running Arista EOS.  It allows playbooks to add or remote
    banner text from the active running configuration.
extends_documentation_fragment: eos
notes:
  - Tested against EOS 4.15
options:
  banner:
    description:
      - Specifies which banner that should be
        configured on the remote device.
    required: true
    choices: ['login', 'motd']
  text:
    description:
      - The banner text that should be
        present in the remote device running configuration.  This argument
        accepts a multiline string. Requires I(state=present).
  state:
    description:
      - Specifies whether or not the configuration is
        present in the current devices active running configuration.
    default: present
    choices: ['present', 'absent']
"""

EXAMPLES = """
- name: configure the login banner
  eos_banner:
    banner: login
    text: |
      this is my login banner
      that contains a multiline
      string
    state: present

- name: remove the motd banner
  eos_banner:
    banner: motd
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - banner login
    - this is my login banner
    - that contains a multiline
    - string
    - EOF
session_name:
  description: The EOS config session name used to load the configuration
  returned: if changes
  type: str
  sample: ansible_1479315771
"""
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.eos.eos import load_config, run_commands
from ansible.module_utils.network.eos.eos import eos_argument_spec, check_args
from ansible.module_utils.six import string_types
from ansible.module_utils._text import to_text


def map_obj_to_commands(updates, module):
    commands = list()
    want, have = updates
    state = module.params['state']

    if state == 'absent' and have.get('text'):
        if isinstance(have['text'], string_types):
            commands.append('no banner %s' % module.params['banner'])
        elif have['text'].get('loginBanner') or have['text'].get('motd'):
            commands.append({'cmd': 'no banner %s' % module.params['banner']})

    elif state == 'present':
        if isinstance(have['text'], string_types):
            if want['text'] != have['text']:
                commands.append('banner %s' % module.params['banner'])
                commands.extend(want['text'].strip().split('\n'))
                commands.append('EOF')
        else:
            have_text = have['text'].get('loginBanner') or have['text'].get('motd')
            if have_text:
                have_text = have_text.strip()

            if to_text(want['text']) != have_text or not have_text:
                # For EAPI we need to construct a dict with cmd/input
                # key/values for the banner
                commands.append({'cmd': 'banner %s' % module.params['banner'],
                                 'input': want['text'].strip('\n')})

    return commands


def map_config_to_obj(module):
    output = run_commands(module, ['show banner %s' % module.params['banner']])
    obj = {'banner': module.params['banner'], 'state': 'absent'}
    if output:
        if module.params['transport'] == 'eapi':
            # On EAPI we need to extract the banner text from dict key
            # 'loginBanner'
            if module.params['banner'] == 'login':
                banner_response_key = 'loginBanner'
            else:
                banner_response_key = 'motd'
            if isinstance(output[0], dict) and banner_response_key in output[0].keys():
                obj['text'] = output[0]
        else:
            obj['text'] = output[0]
        obj['state'] = 'present'
    return obj


def map_params_to_obj(module):
    text = module.params['text']
    if text:
        text = to_text(text).strip()

    return {
        'banner': module.params['banner'],
        'text': text,
        'state': module.params['state']
    }


def main():
    """ main entry point for module execution
    """
    argument_spec = dict(
        banner=dict(required=True, choices=['login', 'motd']),
        text=dict(),
        state=dict(default='present', choices=['present', 'absent'])
    )

    argument_spec.update(eos_argument_spec)

    required_if = [('state', 'present', ('text',))]

    module = AnsibleModule(argument_spec=argument_spec,
                           required_if=required_if,
                           supports_check_mode=True)

    warnings = list()
    check_args(module, warnings)

    result = {'changed': False}
    if warnings:
        result['warnings'] = warnings
    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands((want, have), module)
    result['commands'] = commands

    if commands:
        commit = not module.check_mode
        response = load_config(module, commands, commit=commit)
        if response.get('diff') and module._diff:
            result['diff'] = {'prepared': response.get('diff')}
        result['session_name'] = response.get('session')
        result['changed'] = True

    module.exit_json(**result)

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2012, Matt Wright <matt@nobien.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'core'}

DOCUMENTATION = '''
---
module: pip
short_description: Manages Python library dependencies
description:
     - "Manage Python library dependencies. To use this module, one of the following keys is required: C(name)
       or C(requirements)."
version_added: "0.7"
options:
  name:
    description:
      - The name of a Python library to install or the url of the remote package.
      - As of 2.2 you can supply a list of names.
  version:
    description:
      - The version number to install of the Python library specified in the I(name) parameter.
  requirements:
    description:
      - The path to a pip requirements file, which should be local to the remote system.
        File can be specified as a relative path if using the chdir option.
  virtualenv:
    description:
      - An optional path to a I(virtualenv) directory to install into.
        It cannot be specified together with the 'executable' parameter
        (added in 2.1).
        If the virtualenv does not exist, it will be created before installing
        packages. The optional virtualenv_site_packages, virtualenv_command,
        and virtualenv_python options affect the creation of the virtualenv.
  virtualenv_site_packages:
    description:
      - Whether the virtual environment will inherit packages from the
        global site-packages directory.  Note that if this setting is
        changed on an already existing virtual environment it will not
        have any effect, the environment must be deleted and newly
        created.
    type: bool
    default: "no"
    version_added: "1.0"
  virtualenv_command:
    description:
      - The command or a pathname to the command to create the virtual
        environment with. For example C(pyvenv), C(virtualenv),
        C(virtualenv2), C(~/bin/virtualenv), C(/usr/local/bin/virtualenv).
    default: virtualenv
    version_added: "1.1"
  virtualenv_python:
    description:
      - The Python executable used for creating the virtual environment.
        For example C(python3.5), C(python2.7). When not specified, the
        Python version used to run the ansible module is used. This parameter
        should not be used when C(virtualenv_command) is using C(pyvenv) or
        the C(-m venv) module.
    version_added: "2.0"
  state:
    description:
      - The state of module
      - The 'forcereinstall' option is only available in Ansible 2.1 and above.
    choices: [ absent, forcereinstall, latest, present ]
    default: present
  extra_args:
    description:
      - Extra arguments passed to pip.
    version_added: "1.0"
  editable:
    description:
      - Pass the editable flag.
    type: bool
    default: 'no'
    version_added: "2.0"
  chdir:
    description:
      - cd into this directory before running the command
    version_added: "1.3"
  executable:
    description:
      - The explicit executable or a pathname to the executable to be used to
        run pip for a specific version of Python installed in the system. For
        example C(pip-3.3), if there are both Python 2.7 and 3.3 installations
        in the system and you want to run pip for the Python 3.3 installation.
        It cannot be specified together with the 'virtualenv' parameter (added in 2.1).
        By default, it will take the appropriate version for the python interpreter
        use by ansible, e.g. pip3 on python 3, and pip2 or pip on python 2.
    version_added: "1.3"
  umask:
    description:
      - The system umask to apply before installing the pip package. This is
        useful, for example, when installing on systems that have a very
        restrictive umask by default (e.g., 0077) and you want to pip install
        packages which are to be used by all users. Note that this requires you
        to specify desired umask mode in octal, with a leading 0 (e.g., 0077).
    version_added: "2.1"
notes:
   - Please note that virtualenv (U(http://www.virtualenv.org/)) must be
     installed on the remote host if the virtualenv parameter is specified and
     the virtualenv needs to be created.
   - By default, this module will use the appropriate version of pip for the
     interpreter used by ansible (e.g. pip3 when using python 3, pip2 otherwise)
requirements:
- pip
- virtualenv
author:
- Matt Wright (@mattupstate)
'''

EXAMPLES = '''
# Install (Bottle) python package.
- pip:
    name: bottle

# Install (Bottle) python package on version 0.11.
- pip:
    name: bottle
    version: 0.11

# Install (MyApp) using one of the remote protocols (bzr+,hg+,git+,svn+). You do not have to supply '-e' option in extra_args.
- pip:
    name: svn+http://myrepo/svn/MyApp#egg=MyApp

# Install MyApp using one of the remote protocols (bzr+,hg+,git+).
- pip:
    name: git+http://myrepo/app/MyApp

# Install (MyApp) from local tarball
- pip:
    name: file:///path/to/MyApp.tar.gz

# Install (Bottle) into the specified (virtualenv), inheriting none of the globally installed modules
- pip:
    name: bottle
    virtualenv: /my_app/venv

# Install (Bottle) into the specified (virtualenv), inheriting globally installed modules
- pip:
    name: bottle
    virtualenv: /my_app/venv
    virtualenv_site_packages: yes

# Install (Bottle) into the specified (virtualenv), using Python 2.7
- pip:
    name: bottle
    virtualenv: /my_app/venv
    virtualenv_command: virtualenv-2.7

# Install (Bottle) within a user home directory.
- pip:
    name: bottle
    extra_args: --user

# Install specified python requirements.
- pip:
    requirements: /my_app/requirements.txt

# Install specified python requirements in indicated (virtualenv).
- pip:
    requirements: /my_app/requirements.txt
    virtualenv: /my_app/venv

# Install specified python requirements and custom Index URL.
- pip:
    requirements: /my_app/requirements.txt
    extra_args: -i https://example.com/pypi/simple

# Install (Bottle) for Python 3.3 specifically,using the 'pip-3.3' executable.
- pip:
    name: bottle
    executable: pip-3.3

# Install (Bottle), forcing reinstallation if it's already installed
- pip:
    name: bottle
    state: forcereinstall

# Install (Bottle) while ensuring the umask is 0022 (to ensure other users can use it)
- pip:
    name: bottle
    umask: 0022
  become: True
'''

import os
import re
import sys
import tempfile

from ansible.module_utils.basic import AnsibleModule, is_executable
from ansible.module_utils._text import to_native
from ansible.module_utils.six import PY3


#: Python one-liners to be run at the command line that will determine the
# installed version for these special libraries.  These are libraries that
# don't end up in the output of pip freeze.
_SPECIAL_PACKAGE_CHECKERS = {'setuptools': 'import setuptools; print(setuptools.__version__)',
                             'pip': 'import pkg_resources; print(pkg_resources.get_distribution("pip").version)'}


def _get_cmd_options(module, cmd):
    thiscmd = cmd + " --help"
    rc, stdout, stderr = module.run_command(thiscmd)
    if rc != 0:
        module.fail_json(msg="Could not get output from %s: %s" % (thiscmd, stdout + stderr))

    words = stdout.strip().split()
    cmd_options = [x for x in words if x.startswith('--')]
    return cmd_options


def _get_full_name(name, version=None):
    if version is None:
        resp = name
    else:
        resp = name + '==' + version
    return resp


def _get_packages(module, pip, chdir):
    '''Return results of pip command to get packages.'''
    # Try 'pip list' command first.
    command = '%s list --format=freeze' % pip
    lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
    rc, out, err = module.run_command(command, cwd=chdir, environ_update=lang_env)

    # If there was an error (pip version too old) then use 'pip freeze'.
    if rc != 0:
        command = '%s freeze' % pip
        rc, out, err = module.run_command(command, cwd=chdir)
        if rc != 0:
            _fail(module, command, out, err)

    return (command, out, err)


def _is_present(name, version, installed_pkgs, pkg_command):
    '''Return whether or not package is installed.'''
    for pkg in installed_pkgs:
        if '==' in pkg:
            pkg_name, pkg_version = pkg.split('==')
        else:
            continue

        if pkg_name == name and (version is None or version == pkg_version):
            return True

    return False


def _get_pip(module, env=None, executable=None):
    # Older pip only installed under the "/usr/bin/pip" name.  Many Linux
    # distros install it there.
    # By default, we try to use pip required for the current python
    # interpreter, so people can use pip to install modules dependencies
    candidate_pip_basenames = ('pip2', 'pip')
    if PY3:
        # pip under python3 installs the "/usr/bin/pip3" name
        candidate_pip_basenames = ('pip3',)

    pip = None
    if executable is not None:
        if os.path.isabs(executable):
            pip = executable
        else:
            # If you define your own executable that executable should be the only candidate.
            # As noted in the docs, executable doesn't work with virtualenvs.
            candidate_pip_basenames = (executable,)

    if pip is None:
        if env is None:
            opt_dirs = []
            for basename in candidate_pip_basenames:
                pip = module.get_bin_path(basename, False, opt_dirs)
                if pip is not None:
                    break
            else:
                # For-else: Means that we did not break out of the loop
                # (therefore, that pip was not found)
                module.fail_json(msg='Unable to find any of %s to use.  pip'
                                     ' needs to be installed.' % ', '.join(candidate_pip_basenames))
        else:
            # If we're using a virtualenv we must use the pip from the
            # virtualenv
            venv_dir = os.path.join(env, 'bin')
            candidate_pip_basenames = (candidate_pip_basenames[0], 'pip')
            for basename in candidate_pip_basenames:
                candidate = os.path.join(venv_dir, basename)
                if os.path.exists(candidate) and is_executable(candidate):
                    pip = candidate
                    break
            else:
                # For-else: Means that we did not break out of the loop
                # (therefore, that pip was not found)
                module.fail_json(msg='Unable to find pip in the virtualenv, %s, ' % env +
                                     'under any of these names: %s. ' % (', '.join(candidate_pip_basenames)) +
                                     'Make sire pip is present in the virtualenv.')

    return pip


def _fail(module, cmd, out, err):
    msg = ''
    if out:
        msg += "stdout: %s" % (out, )
    if err:
        msg += "\n:stderr: %s" % (err, )
    module.fail_json(cmd=cmd, msg=msg)


def _get_package_info(module, package, env=None):
    """This is only needed for special packages which do not show up in pip freeze

    pip and setuptools fall into this category.

    :returns: a string containing the version number if the package is
        installed.  None if the package is not installed.
    """
    if env:
        opt_dirs = ['%s/bin' % env]
    else:
        opt_dirs = []
    python_bin = module.get_bin_path('python', False, opt_dirs)

    if python_bin is None:
        formatted_dep = None
    else:
        rc, out, err = module.run_command([python_bin, '-c', _SPECIAL_PACKAGE_CHECKERS[package]])
        if rc:
            formatted_dep = None
        else:
            formatted_dep = '%s==%s' % (package, out.strip())
    return formatted_dep


def setup_virtualenv(module, env, chdir, out, err):
    if module.check_mode:
        module.exit_json(changed=True)

    cmd = module.params['virtualenv_command']
    if os.path.basename(cmd) == cmd:
        cmd = module.get_bin_path(cmd, True)

    if module.params['virtualenv_site_packages']:
        cmd += ' --system-site-packages'
    else:
        cmd_opts = _get_cmd_options(module, cmd)
        if '--no-site-packages' in cmd_opts:
            cmd += ' --no-site-packages'

    virtualenv_python = module.params['virtualenv_python']
    # -p is a virtualenv option, not compatible with pyenv or venv
    # this if validates if the command being used is not any of them
    if not any(ex in module.params['virtualenv_command'] for ex in ('pyvenv', '-m venv')):
        if virtualenv_python:
            cmd += ' -p%s' % virtualenv_python
        elif PY3:
            # Ubuntu currently has a patch making virtualenv always
            # try to use python2.  Since Ubuntu16 works without
            # python2 installed, this is a problem.  This code mimics
            # the upstream behaviour of using the python which invoked
            # virtualenv to determine which python is used inside of
            # the virtualenv (when none are specified).
            cmd += ' -p%s' % sys.executable

    # if venv or pyvenv are used and virtualenv_python is defined, then
    # virtualenv_python is ignored, this has to be acknowledged
    elif module.params['virtualenv_python']:
        module.fail_json(
            msg='virtualenv_python should not be used when'
                ' using the venv module or pyvenv as virtualenv_command'
        )

    cmd = "%s %s" % (cmd, env)
    rc, out_venv, err_venv = module.run_command(cmd, cwd=chdir)
    out += out_venv
    err += err_venv
    if rc != 0:
        _fail(module, cmd, out, err)
    return out, err


def main():
    state_map = dict(
        present='install',
        absent='uninstall -y',
        latest='install -U',
        forcereinstall='install -U --force-reinstall',
    )

    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type='str', default='present', choices=state_map.keys()),
            name=dict(type='list'),
            version=dict(type='str'),
            requirements=dict(type='str'),
            virtualenv=dict(type='path'),
            virtualenv_site_packages=dict(type='bool', default=False),
            virtualenv_command=dict(type='path', default='virtualenv'),
            virtualenv_python=dict(type='str'),
            use_mirrors=dict(type='bool', default=True),
            extra_args=dict(type='str'),
            editable=dict(type='bool', default=False),
            chdir=dict(type='path'),
            executable=dict(type='path'),
            umask=dict(type='str'),
        ),
        required_one_of=[['name', 'requirements']],
        mutually_exclusive=[['name', 'requirements'], ['executable', 'virtualenv']],
        supports_check_mode=True,
    )

    state = module.params['state']
    name = module.params['name']
    version = module.params['version']
    requirements = module.params['requirements']
    extra_args = module.params['extra_args']
    chdir = module.params['chdir']
    umask = module.params['umask']
    env = module.params['virtualenv']

    venv_created = False
    if chdir:
        env = os.path.join(chdir, env)

    if umask and not isinstance(umask, int):
        try:
            umask = int(umask, 8)
        except Exception:
            module.fail_json(msg="umask must be an octal integer",
                             details=to_native(sys.exc_info()[1]))

    old_umask = None
    if umask is not None:
        old_umask = os.umask(umask)
    try:
        if state == 'latest' and version is not None:
            module.fail_json(msg='version is incompatible with state=latest')

        if chdir is None:
            # this is done to avoid permissions issues with privilege escalation and virtualenvs
            chdir = tempfile.gettempdir()

        err = ''
        out = ''

        if env:
            if not os.path.exists(os.path.join(env, 'bin', 'activate')):
                venv_created = True
                out, err = setup_virtualenv(module, env, chdir, out, err)

        pip = _get_pip(module, env, module.params['executable'])

        cmd = '%s %s' % (pip, state_map[state])

        # If there's a virtualenv we want things we install to be able to use other
        # installations that exist as binaries within this virtualenv. Example: we
        # install cython and then gevent -- gevent needs to use the cython binary,
        # not just a python package that will be found by calling the right python.
        # So if there's a virtualenv, we add that bin/ to the beginning of the PATH
        # in run_command by setting path_prefix here.
        path_prefix = None
        if env:
            path_prefix = "/".join(pip.split('/')[:-1])

        # Automatically apply -e option to extra_args when source is a VCS url. VCS
        # includes those beginning with svn+, git+, hg+ or bzr+
        has_vcs = False
        if name:
            for pkg in name:
                if bool(pkg and re.match(r'(svn|git|hg|bzr)\+', pkg)):
                    has_vcs = True
                    break

        if module.params['editable']:
            args_list = []  # used if extra_args is not used at all
            if extra_args:
                args_list = extra_args.split(' ')
            if '-e' not in args_list:
                args_list.append('-e')
                # Ok, we will reconstruct the option string
                extra_args = ' '.join(args_list)

        if extra_args:
            cmd += ' %s' % extra_args

        if name:
            for pkg in name:
                cmd += ' %s' % _get_full_name(pkg, version)
        elif requirements:
            cmd += ' -r %s' % requirements
        else:
            module.exit_json(
                changed=False,
                warnings=["No valid name or requirements file found."],
            )

        if module.check_mode:
            if extra_args or requirements or state == 'latest' or not name:
                module.exit_json(changed=True)

            pkg_cmd, out_pip, err_pip = _get_packages(module, pip, chdir)

            out += out_pip
            err += err_pip

            changed = False
            if name:
                pkg_list = [p for p in out.split('\n') if not p.startswith('You are using') and not p.startswith('You should consider') and p]

                if pkg_cmd.endswith(' freeze') and ('pip' in name or 'setuptools' in name):
                    # Older versions of pip (pre-1.3) do not have pip list.
                    # pip freeze does not list setuptools or pip in its output
                    # So we need to get those via a specialcase
                    for pkg in ('setuptools', 'pip'):
                        if pkg in name:
                            formatted_dep = _get_package_info(module, pkg, env)
                            if formatted_dep is not None:
                                pkg_list.append(formatted_dep)
                                out += '%s\n' % formatted_dep

                for pkg in name:
                    is_present = _is_present(pkg, version, pkg_list, pkg_cmd)
                    if (state == 'present' and not is_present) or (state == 'absent' and is_present):
                        changed = True
                        break
            module.exit_json(changed=changed, cmd=pkg_cmd, stdout=out, stderr=err)

        out_freeze_before = None
        if requirements or has_vcs:
            _, out_freeze_before, _ = _get_packages(module, pip, chdir)

        rc, out_pip, err_pip = module.run_command(cmd, path_prefix=path_prefix, cwd=chdir)
        out += out_pip
        err += err_pip
        if rc == 1 and state == 'absent' and \
           ('not installed' in out_pip or 'not installed' in err_pip):
            pass  # rc is 1 when attempting to uninstall non-installed package
        elif rc != 0:
            _fail(module, cmd, out, err)

        if state == 'absent':
            changed = 'Successfully uninstalled' in out_pip
        else:
            if out_freeze_before is None:
                changed = 'Successfully installed' in out_pip
            else:
                _, out_freeze_after, _ = _get_packages(module, pip, chdir)
                changed = out_freeze_before != out_freeze_after

        changed = changed or venv_created

        module.exit_json(changed=changed, cmd=cmd, name=name, version=version,
                         state=state, requirements=requirements, virtualenv=env,
                         stdout=out, stderr=err)
    finally:
        if old_umask is not None:
            os.umask(old_umask)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['deprecated'],
                    'supported_by': 'certified'}


DOCUMENTATION = '''
---
module: ec2_vpc
short_description: configure AWS virtual private clouds
description:
    - Create or terminates AWS virtual private clouds.  This module has a dependency on python-boto.
version_added: "1.4"
deprecated:
  removed_in: "2.5"
  why: Replaced by dedicated modules.
  alternative: Use M(ec2_vpc_net) along with supporting modules including M(ec2_vpc_igw), M(ec2_vpc_route_table), M(ec2_vpc_subnet),
               M(ec2_vpc_dhcp_option), M(ec2_vpc_nat_gateway), M(ec2_vpc_nacl).
options:
  cidr_block:
    description:
      - "The cidr block representing the VPC, e.g. C(10.0.0.0/16), required when I(state=present)."
  instance_tenancy:
    description:
      - "The supported tenancy options for instances launched into the VPC."
    default: "default"
    choices: [ "default", "dedicated" ]
  dns_support:
    description:
      - Toggles the "Enable DNS resolution" flag.
    type: bool
    default: 'yes'
  dns_hostnames:
    description:
      - Toggles the "Enable DNS hostname support for instances" flag.
    type: bool
    default: 'yes'
  subnets:
    description:
      - 'A dictionary array of subnets to add of the form C({ cidr: ..., az: ... , resource_tags: ... }).'
      - Where C(az) is the desired availability zone of the subnet, optional.
      - 'Tags C(resource_tags) use dictionary form C({ "Environment":"Dev", "Tier":"Web", ...}), optional.'
      - C(resource_tags) see resource_tags for VPC below. The main difference is subnet tags not specified here will be deleted.
      - All VPC subnets not in this list will be removed as well.
      - As of 1.8, if the subnets parameter is not specified, no existing subnets will be modified.'
  vpc_id:
    description:
      - A VPC id to terminate when I(state=absent).
  resource_tags:
    description:
      - 'A dictionary array of resource tags of the form C({ tag1: value1, tag2: value2 }).
      - Tags in this list are used in conjunction with CIDR block to uniquely identify a VPC in lieu of vpc_id. Therefore,
        if CIDR/Tag combination does not exist, a new VPC will be created.  VPC tags not on this list will be ignored. Prior to 1.7,
        specifying a resource tag was optional.'
    required: true
    version_added: "1.6"
  internet_gateway:
    description:
      - Toggle whether there should be an Internet gateway attached to the VPC.
    type: bool
    default: 'no'
  route_tables:
    description:
      - >
        A dictionary array of route tables to add of the form:
        C({ subnets: [172.22.2.0/24, 172.22.3.0/24,], routes: [{ dest: 0.0.0.0/0, gw: igw},], resource_tags: ... }). Where the subnets list is
        those subnets the route table should be associated with, and the routes list is a list of routes to be in the table.  The special keyword
        for the gw of igw specifies that you should the route should go through the internet gateway attached to the VPC. gw also accepts instance-ids,
        interface-ids, and vpc-peering-connection-ids in addition igw. resource_tags is optional and uses dictionary form: C({ "Name": "public", ... }).
        This module is currently unable to affect the "main" route table due to some limitations in boto, so you must explicitly define the associated
        subnets or they will be attached to the main table implicitly. As of 1.8, if the route_tables parameter is not specified, no existing routes
        will be modified.
  wait:
    description:
      - Wait for the VPC to be in state 'available' before returning.
    type: bool
    default: 'no'
  wait_timeout:
    description:
      - How long before wait gives up, in seconds.
    default: 300
  state:
    description:
      - Create or terminate the VPC.
    required: true
    choices: [ "present", "absent" ]
author: "Carson Gee (@carsongee)"
extends_documentation_fragment:
    - aws
    - ec2
'''

EXAMPLES = '''
# Note: None of these examples set aws_access_key, aws_secret_key, or region.
# It is assumed that their matching environment variables are set.

# Basic creation example:
    - ec2_vpc:
        state: present
        cidr_block: 172.23.0.0/16
        resource_tags: { "Environment":"Development" }
        region: us-west-2
# Full creation example with subnets and optional availability zones.
# The absence or presence of subnets deletes or creates them respectively.
    - ec2_vpc:
        state: present
        cidr_block: 172.22.0.0/16
        resource_tags: { "Environment":"Development" }
        subnets:
          - cidr: 172.22.1.0/24
            az: us-west-2c
            resource_tags: { "Environment":"Dev", "Tier" : "Web" }
          - cidr: 172.22.2.0/24
            az: us-west-2b
            resource_tags: { "Environment":"Dev", "Tier" : "App" }
          - cidr: 172.22.3.0/24
            az: us-west-2a
            resource_tags: { "Environment":"Dev", "Tier" : "DB" }
        internet_gateway: True
        route_tables:
          - subnets:
              - 172.22.2.0/24
              - 172.22.3.0/24
            routes:
              - dest: 0.0.0.0/0
                gw: igw
          - subnets:
              - 172.22.1.0/24
            routes:
              - dest: 0.0.0.0/0
                gw: igw
        region: us-west-2
      register: vpc

# Removal of a VPC by id
    - ec2_vpc:
        state: absent
        vpc_id: vpc-aaaaaaa
        region: us-west-2
# If you have added elements not managed by this module, e.g. instances, NATs, etc then
# the delete will fail until those dependencies are removed.
'''

from ansible.module_utils.common.removed import removed_module

if __name__ == '__main__':
    removed_module()



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
#
# Copyright (C) 2017 Lenovo, Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
# Module to send BGP commands to Lenovo Switches
# Lenovo Networking
#

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
---
module: cnos_bgp
author: "Anil Kumar Muraleedharan (@amuraleedhar)"
short_description: Manage BGP resources and attributes on devices running Lenovo CNOS
description:
    - This module allows you to work with Border Gateway Protocol (BGP) related configurations.
     The operators used are overloaded to ensure control over switch BGP configurations. This
     module is invoked using method with asNumber as one of its arguments. The first level of
     the BGP configuration allows to set up an AS number, with the following attributes going
     into various configuration operations under the context of BGP. After passing this level,
     there are eight BGP arguments that will perform further configurations. They are bgpArg1,
     bgpArg2, bgpArg3, bgpArg4, bgpArg5, bgpArg6, bgpArg7, and bgpArg8. For more details on
     how to use these arguments, see [Overloaded Variables].
     This module uses SSH to manage network device configuration.
     The results of the operation will be placed in a directory named 'results'
     that must be created by the user in their local directory to where the playbook is run.
     For more information about this module from Lenovo and customizing it usage for your
     use cases, please visit U(http://systemx.lenovofiles.com/help/index.jsp?topic=%2Fcom.lenovo.switchmgt.ansible.doc%2Fcnos_bgp.html)
version_added: "2.3"
extends_documentation_fragment: cnos
options:
    asNum:
        description:
            - AS number
        required: Yes
        default: Null
    bgpArg1:
        description:
            - This is an overloaded bgp first argument. Usage of this argument can be found is the User Guide referenced above.
        required: Yes
        default: Null
        choices: [address-family,bestpath,bgp,cluster-id,confederation,enforce-first-as,fast-external-failover,
        graceful-restart,graceful-restart-helper,log-neighbor-changes,maxas-limit,neighbor,router-id,shutdown,
        synchronization,timers,vrf]
    bgpArg2:
        description:
            - This is an overloaded bgp second argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [ipv4 or ipv6, always-compare-med,compare-confed-aspath,compare-routerid,dont-compare-originator-id,tie-break-on-age,
        as-path,med,identifier,peers]
    bgpArg3:
        description:
            - This is an overloaded bgp third argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [aggregate-address,client-to-client,dampening,distance,maximum-paths,network,nexthop,redistribute,save,synchronization,
        ignore or multipath-relax, confed or missing-as-worst or non-deterministic or remove-recv-med or remove-send-med]
    bgpArg4:
        description:
            - This is an overloaded bgp fourth argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [Aggregate prefix, Reachability Half-life time,route-map, Distance for routes external,ebgp or ibgp,
        IP prefix <network>,IP prefix <network>/<length>, synchronization, Delay value, direct, ospf, static, memory]
    bgpArg5:
        description:
            - This is an overloaded bgp fifth argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [as-set, summary-only, Value to start reusing a route, Distance for routes internal, Supported multipath numbers,
        backdoor, map, route-map ]
    bgpArg6:
        description:
            - This is an overloaded bgp sixth argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [summary-only,as-set, route-map name, Value to start suppressing a route, Distance for local routes,  Network mask,
        Pointer to route-map entries]
    bgpArg7:
        description:
            - This is an overloaded bgp seventh argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [ Maximum duration to suppress a stable route(minutes), backdoor,route-map, Name of the route map ]
    bgpArg8:
        description:
            - This is an overloaded bgp eigth argument. Usage of this argument can be found is the User Guide referenced above.
        required: No
        default: Null
        choices: [ Un-reachability Half-life time for the penalty(minutes), backdoor]
'''
EXAMPLES = '''
Tasks: The following are examples of using the module cnos_bgp. These are written in the main.yml file of the tasks directory.
---
- name: Test BGP  - neighbor
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "neighbor"
      bgpArg2: "10.241.107.40"
      bgpArg3: 13
      bgpArg4: "address-family"
      bgpArg5: "ipv4"
      bgpArg6: "next-hop-self"

- name: Test BGP  - BFD
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "neighbor"
      bgpArg2: "10.241.107.40"
      bgpArg3: 13
      bgpArg4: "bfd"

- name: Test BGP  - address-family - dampening
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "address-family"
      bgpArg2: "ipv4"
      bgpArg3: "dampening"
      bgpArg4: 13
      bgpArg5: 233
      bgpArg6: 333
      bgpArg7: 15
      bgpArg8: 33

- name: Test BGP  - address-family - network
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "address-family"
      bgpArg2: "ipv4"
      bgpArg3: "network"
      bgpArg4: "1.2.3.4/5"
      bgpArg5: "backdoor"

- name: Test BGP - bestpath - always-compare-med
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "bestpath"
      bgpArg2: "always-compare-med"

- name: Test BGP - bestpath-compare-confed-aspat
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "bestpath"
      bgpArg2: "compare-confed-aspath"

- name: Test BGP - bgp
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "bgp"
      bgpArg2: 33

- name: Test BGP  - cluster-id
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "cluster-id"
      bgpArg2: "1.2.3.4"

- name: Test BGP - confederation-identifier
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "confederation"
      bgpArg2: "identifier"
      bgpArg3: 333

- name: Test BGP - enforce-first-as
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "enforce-first-as"

- name: Test BGP - fast-external-failover
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "fast-external-failover"

- name: Test BGP  - graceful-restart
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "graceful-restart"
      bgpArg2: 333

- name: Test BGP - graceful-restart-helper
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "graceful-restart-helper"

- name: Test BGP - maxas-limit
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "maxas-limit"
      bgpArg2: 333

- name: Test BGP  - neighbor
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "neighbor"
      bgpArg2: "10.241.107.40"
      bgpArg3: 13
      bgpArg4: "address-family"
      bgpArg5: "ipv4"
      bgpArg6: "next-hop-self"

- name: Test BGP - router-id
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "router-id"
      bgpArg2: "1.2.3.4"

- name: Test BGP - synchronization
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "synchronization"

- name: Test BGP - timers
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "timers"
      bgpArg2: 333
      bgpArg3: 3333

- name: Test BGP - vrf
  cnos_bgp:
      host: "{{ inventory_hostname }}"
      username: "{{ hostvars[inventory_hostname]['ansible_ssh_user'] }}"
      password: "{{ hostvars[inventory_hostname]['ansible_ssh_pass'] }}"
      deviceType: "{{ hostvars[inventory_hostname]['deviceType'] }}"
      enablePassword: "{{ hostvars[inventory_hostname]['enablePassword'] }}"
      outputfile: "./results/test_bgp_{{ inventory_hostname }}_output.txt"
      asNum: 33
      bgpArg1: "vrf"

'''
RETURN = '''
msg:
  description: Success or failure message. Upon any failure, the method returns an error display string.
  returned: always
  type: string
'''

import sys
try:
    import paramiko
    HAS_PARAMIKO = True
except ImportError:
    HAS_PARAMIKO = False
import time
import socket
import array
import json
import time
import re
try:
    from ansible.module_utils.network.cnos import cnos
    HAS_LIB = True
except:
    HAS_LIB = False
from ansible.module_utils.basic import AnsibleModule
from collections import defaultdict


def main():
    module = AnsibleModule(
        argument_spec=dict(
            outputfile=dict(required=True),
            host=dict(required=True),
            username=dict(required=True),
            password=dict(required=True, no_log=True),
            enablePassword=dict(required=False, no_log=True),
            deviceType=dict(required=True),
            bgpArg1=dict(required=True),
            bgpArg2=dict(required=False),
            bgpArg3=dict(required=False),
            bgpArg4=dict(required=False),
            bgpArg5=dict(required=False),
            bgpArg6=dict(required=False),
            bgpArg7=dict(required=False),
            bgpArg8=dict(required=False),
            asNum=dict(required=True),),
        supports_check_mode=False)

    username = module.params['username']
    password = module.params['password']
    enablePassword = module.params['enablePassword']
    bgpArg1 = module.params['bgpArg1']
    bgpArg2 = module.params['bgpArg2']
    bgpArg3 = module.params['bgpArg3']
    bgpArg4 = module.params['bgpArg4']
    bgpArg5 = module.params['bgpArg5']
    bgpArg6 = module.params['bgpArg6']
    bgpArg7 = module.params['bgpArg7']
    bgpArg8 = module.params['bgpArg8']
    asNum = module.params['asNum']
    outputfile = module.params['outputfile']
    hostIP = module.params['host']
    deviceType = module.params['deviceType']
    output = ""

    if not HAS_PARAMIKO:
        module.fail_json(msg='paramiko is required for this module')
    # Create instance of SSHClient object
    remote_conn_pre = paramiko.SSHClient()

    # Automatically add untrusted hosts (make sure okay for security policy in your environment)
    remote_conn_pre.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    # initiate SSH connection with the switch
    remote_conn_pre.connect(hostIP, username=username, password=password)
    time.sleep(2)

    # Use invoke_shell to establish an 'interactive session'
    remote_conn = remote_conn_pre.invoke_shell()
    time.sleep(2)

    # Enable and enter configure terminal then send command
    output = output + cnos.waitForDeviceResponse("\n", ">", 2, remote_conn)

    output = output + cnos.enterEnableModeForDevice(enablePassword, 3, remote_conn)

    # Make terminal length = 0
    output = output + cnos.waitForDeviceResponse("terminal length 0\n", "#", 2, remote_conn)

    # Go to config mode
    output = output + cnos.waitForDeviceResponse("configure device\n", "(config)#", 2, remote_conn)

    # Send the CLi command
    output = output + cnos.routerConfig(remote_conn, deviceType, "(config)#", 2, "bgp", asNum,
                                        bgpArg1, bgpArg2, bgpArg3, bgpArg4, bgpArg5, bgpArg6, bgpArg7, bgpArg8)

    # Save it into the file
    file = open(outputfile, "a")
    file.write(output)
    file.close()

    # Logic to check when changes occur or not
    errorMsg = cnos.checkOutputForError(output)
    if(errorMsg is None):
        module.exit_json(changed=True, msg="BGP configurations accomplished")
    else:
        module.fail_json(msg=errorMsg)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright 2017, Ansible Project
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: logentries_msg
version_added: "2.5"
short_description: Send a message to logentries.
description:
   - Send a message to logentries
requirements:
  - "python >= 2.6"
options:
  token:
    description:
      - Log token.
    required: true
  msg:
    description:
      - The message body.
    required: true
  api:
    description:
      - API endpoint
    default: data.logentries.com
  port:
    description:
      - API endpoint port
    default: 80
author: "Jimmy Tang (@jcftang) <jimmy_tang@rapid7.com>"
'''

RETURN = '''# '''

EXAMPLES = '''
- logentries_msg:
    token=00000000-0000-0000-0000-000000000000
    msg="{{ ansible_hostname }}"
'''

import socket

from ansible.module_utils.basic import AnsibleModule


def send_msg(module, token, msg, api, port):

    message = "{} {}\n".format(token, msg)

    api_ip = socket.gethostbyname(api)

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((api_ip, port))
    try:
        if not module.check_mode:
            s.send(message)
    except Exception as e:
        module.fail_json(msg="failed to send message, msg=%s" % e)
    s.close()


def main():
    module = AnsibleModule(
        argument_spec=dict(
            token=dict(type='str', required=True),
            msg=dict(type='str', required=True),
            api=dict(type='str', default="data.logentries.com"),
            port=dict(type='int', default=80)),
        supports_check_mode=True
    )

    token = module.params["token"]
    msg = module.params["msg"]
    api = module.params["api"]
    port = module.params["port"]

    changed = False
    try:
        send_msg(module, token, msg, api, port)
        changed = True
    except Exception as e:
        module.fail_json(msg="unable to send msg: %s" % e)

    module.exit_json(changed=changed, msg=msg)


if __name__ == '__main__':
    main()
